import math
import re

import numpy as np

from BluPrintTriboSys import TriboSys
from Constants import PltOpts, SubDir, TexTempl, UnitTex, PrintOpts, Unit, \
    PreSol
from cartesian_plot_functions import plt_contact, plt_profile, plt_3d, plt_2d, \
    plt_2d_scatt_line, plt_2d_2y_ax, \
    plt_profile_approx
from generate_latex_output import get_calc_specific_latex_template
from hertz_equations import hertz_displ
from influ_matrix_management import cache_influ_mat, load_influ_mat
from solve_half_space import solve_half_space, pre_solve_half_space
from system_functions import print_it, to_preci, save_to_matlab


class BallOnDisk(TriboSys):
    """Global tribosystem data"""

    def __init__(self, num_balls, sliding_diam, ball, disk, global_force,
                 setup_name='ball-on-disk'):
        super().__init__(setup_name, global_force, None)

        self.num_balls = num_balls
        self.sliding_diam = sliding_diam  # in mm
        self.ball = ball
        self.disk = disk
        self.rot_vel1 = None
        self.effective_rot_vel = None
        self.effective_omega = None
        self.raceway_vel = None
        self.ball.norm_forces = None
        self.ball.press = None
        self.ball.max_press = None
        self.ball.rel_vel = None
        self.pv = None
        self.init_force = None
        self.influ_mat_db_1 = None
        self.slip = None
        self.rel_vel = None
        if self.ball.diameter == self.ball.circle_radius:
            self.ball.is_ball = True
        else:
            self.ball.is_ball = False

    def calc_load_distribution(self, ui=None, res_dir=None):
        """Calculate load distribution"""
        print_it("calculating load distribution")
        self.ball.norm_forces = np.multiply(np.ones(self.num_balls),
                                            self.global_force / self.num_balls)
        self.init_force = self.global_force / self.num_balls

    def get_grid_size(self, ui=None, res_dir=None):
        """Determine grid size by running (quick) simulation with simplified
        contact bodies"""
        print_it('determining grid size', PrintOpts.lvl1.value)
        self.ball.simple_clone()
        self.ball.clone.make_profile(PreSol.res_x.value, PreSol.res_y.value,
                                     self.init_force)
        self.disk.simple_clone()
        self.disk.clone.make_slave_to(self.ball.clone)

        init_displ = hertz_displ(self.ball.clone.e, self.disk.clone.e,
                                 self.ball.clone.ny, self.disk.clone.ny,
                                 self.ball.clone.r_hertz_x,
                                 self.ball.clone.r_hertz_y,
                                 self.disk.clone.r_hertz_x,
                                 self.disk.clone.r_hertz_y,
                                 self.ball.norm_forces[0])
        contact_delta = 1
        contact_width = 0.025
        while contact_delta != 0:
            self.ball.clone.make_profile(PreSol.res_x.value, PreSol.res_y.value,
                                         self.init_force,
                                         contact_width=contact_width)
            self.disk.clone.make_slave_to(self.ball)

            pressure, init_displ = \
                pre_solve_half_space(self.ball.clone.profile,
                                     self.disk.clone.profile,
                                     self.ball.clone.x_axis,
                                     self.ball.clone.y_axis,
                                     self.ball.clone.res_x,
                                     self.ball.clone.res_y,
                                     self.ball.clone.delta_x,
                                     self.ball.clone.delta_y, self.ball.clone.e,
                                     self.disk.clone.e, self.ball.clone.ny,
                                     self.disk.clone.ny,
                                     self.ball.clone.norm_forces[0],
                                     init_displ=init_displ)

            pressure_elements = sum(
                pressure[math.floor(self.ball.clone.res_y / 2), :] > 0)
            contact_delta = pressure_elements + 2 - self.ball.clone.res_y
            contact_width += np.sign(contact_delta) * contact_width / 25

        self.ball.make_profile(PreSol.res_x.value, PreSol.res_y.value,
                               self.init_force, contact_width=contact_width)
        self.disk.make_slave_to(self.ball)
        return init_displ

    def calc_contact_pressure(self, ui=None, res_dir=None):
        """Calculate contact pressure distribution between ball and disk"""
        print_it('calculating 1 pressure distribution')

        init_displ = self.get_grid_size(ui, res_dir)
        [self.influ_mat_db_1] = load_influ_mat(ui, res_dir, 1)
        print_it('solving first half space', PrintOpts.lvl1.value)
        self.ball.press, self.influ_mat_db_1 = \
            solve_half_space(self.ball.profile, self.disk.profile,
                             self.ball.x_axis, self.ball.y_axis,
                             self.ball.res_x,
                             self.ball.res_y, self.ball.delta_x,
                             self.ball.delta_y, self.ball.e,
                             self.disk.e,
                             self.ball.ny, self.disk.ny,
                             self.ball.norm_forces[0], res_dir,
                             init_displ=init_displ,
                             influ_mat_db=self.influ_mat_db_1)

        cache_influ_mat(ui, [self.influ_mat_db_1], res_dir)
        self.ball.max_press = np.amax(self.ball.press, axis=1)
        self.save_pressure_to_matlab(res_dir)

    def save_pressure_to_matlab(self, res_dir):
        dat_dict = dict(contact_pressure=self.ball.press,
                        x_axis=self.ball.x_axis,
                        y_axis=self.ball.y_axis)
        save_to_matlab(dat_dict, res_dir, 'pressure-field')

    def calc_kinematics(self, rot_vel1, rot_vel2, ui=None, res_dir=None):
        """Calculate tribosystem kinematics based on rotational velocities of
        washer disks"""
        print_it("calculating kinematics")
        self.ball.rot_vel = rot_vel1
        self.disk.rot_vel = rot_vel2
        self.ball.omega = self.ball.rot_vel / 60
        self.disk.omega = self.disk.rot_vel / 60
        self.ball.vel = (
                        self.ball.diameter + self.ball.x_axis) * \
                        math.pi * self.ball.omega
        self.disk.vel = (
                        self.disk.diameter + self.ball.x_axis) * \
                        math.pi * self.disk.omega
        self.slip = (self.ball.vel - self.disk.vel) / self.ball.vel
        self.rel_vel = np.ones(self.ball.res_x) * (
        self.ball.vel - self.disk.vel)

        self.ball.footpr_vel = 2 * math.pi * self.ball.diameter / 2 * \
                               self.ball.omega
        self.disk.footpr_vel = 2 * math.pi * self.disk.diameter / 2 * \
                               self.disk.omega
        self.ball.overroll_t_incr = self.ball.delta_y / self.ball.footpr_vel
        self.disk.overroll_t_incr = self.disk.delta_y / self.disk.footpr_vel

        self.ball.overroll_t = np.divide(self.ball.press_zone_len,
                                         self.ball.footpr_vel)
        self.disk.overroll_t = np.divide(self.ball.press_zone_len,
                                         self.disk.footpr_vel)
        self.ball.no_overroll_t = np.divide(
            (2 * math.pi * (self.ball.diameter / 2) - self.num_balls *
             self.ball.press_zone_len) / self.num_balls, self.ball.footpr_vel)
        self.disk.no_overroll_t = np.divide(
            (2 * math.pi * (self.disk.diameter / 2) - self.ball.press_zone_len),
            self.disk.footpr_vel)

    def calc_pv(self, ui=None, res_dir=None):
        """Calculate product of local maximum pressure and local maximum
        relative velocity"""
        print_it("calculating pv_rel")
        self.pv = np.multiply(self.ball.max_press, self.rel_vel) / 1000

    def calc_e_akin(self, ui=None, res_dir=None):
        """"Calculate the kinetic friction energy accumulation in W per m^2"""
        print_it("calculating e_a,kin")
        pv_local = np.multiply(self.ball.press.sum(1), self.rel_vel)
        self.ball.e_akin = np.absolute(
            np.divide(np.multiply(pv_local, self.ball.overroll_t_incr),
                      self.ball.no_overroll_t)) / 1000
        self.disk.e_akin = np.absolute(
            np.divide(np.multiply(pv_local, self.disk.overroll_t_incr),
                      self.disk.no_overroll_t)) / 1000

    def plot_it(self, ui=None, res_dir=None):
        """Orchestrate output plot generation"""
        print_it("plotting results and finishing up")
        plt_profile(self.ball, PltOpts.DD.value, res_dir, SubDir.profiles.value)
        plt_profile(self.ball, PltOpts.DDD.value, res_dir,
                    SubDir.profiles.value)
        plt_profile(self.disk, PltOpts.DD.value, res_dir, SubDir.profiles.value)
        plt_profile(self.disk, PltOpts.DDD.value, res_dir,
                    SubDir.profiles.value)
        plt_profile_approx(res_dir, SubDir.profiles.value)
        plt_contact(self.ball, self.disk, PltOpts.DD.value, res_dir,
                    SubDir.contacts.value)
        plt_contact(self.ball, self.disk, PltOpts.DDD.value, res_dir,
                    SubDir.contacts.value)

        plt_3d(self.ball.x_axis, self.ball.y_axis, self.ball.press,
               self.ball.x_label, self.ball.y_label,
               'pressure in MPa', 'contact_pressure_ball', res_dir,
               SubDir.pressures.value,
               'contact_pressure')
        plt_2d(self.ball.x_axis, self.rel_vel, self.ball.x_label, 'rel vel',
               'rel vel', res_dir,
               SubDir.energy.value, 'rel_vel')
        plt_2d(self.ball.x_axis, self.pv, self.ball.x_label,
               'pv_rel in {}'.format(Unit.pvrel.value), 'pv_rel',
               res_dir, SubDir.energy.value, 'pv_rel')
        plt_2d_scatt_line(self.ball.x_axis, self.ball.e_akin, self.ball.x_axis,
                          self.ball.e_akin,
                          self.ball.x_label,
                          'e_akin in {}'.format(Unit.eakin.value),
                          'e_akin ball',
                          res_dir, SubDir.energy.value, 'e-akin-ball')
        plt_2d_2y_ax(self.ball.x_axis, self.ball.e_akin, self.ball.x_axis,
                     self.pv, self.ball.x_label,
                     'e_akin vs pv_rel',
                     'e_akin in {}'.format(Unit.eakin.value),
                     'pv_rel in {}'.format(Unit.pvrel.value), res_dir,
                     SubDir.energy.value,
                     'e_akin_vs_pv_rel ball')

    def generate_latex_output(self, calc_spec_tex_file_handle, sim, ui=None,
                              res_dir=None):
        """Generate the calculation-specific part of the latex output file"""
        average_pressure = np.mean(self.ball.press[self.ball.press > 0])
        numeric_output_data = [
            ('pressure, max.', to_preci(np.amax(self.ball.press), 4),
             UnitTex.pressure.value, 'unverified'),
            ('pressure, av.', to_preci(average_pressure, 4),
             UnitTex.pressure.value, 'unverified'),
            ('e_a,kin ball, max.', to_preci(np.amax(self.ball.e_akin), 4),
             UnitTex.eakin.value, 'unverified'),
            ('e_a,kin disk, max.', to_preci(np.amax(self.disk.e_akin), 4),
             UnitTex.eakin.value, 'unverified'),
            ('pv_rel, max.', to_preci(np.amax(self.pv), 4),
             UnitTex.pvrel.value, 'unverified'),
            ('contact area', to_preci(self.ball.get_area(self.ball.press), 4),
             UnitTex.area.value, 'unverified')]
        table_calc_summary = []
        for key, value, unit, status in sorted(numeric_output_data):
            table_calc_summary.append(
                (re.sub('_', '\_', key), value, unit, status))

        latex_variables = {'table_calc_summary': table_calc_summary,
                           'contact_plot1': '{}/contact1.png'.format(
                               SubDir.tex_figs_rel_to_tex_file.value),
                           'pressure_plot1': '{}/pressure1.png'.format(
                               SubDir.tex_figs_rel_to_tex_file.value),
                           'energy_plot1': '{}/energy1.png'.format(
                               SubDir.tex_figs_rel_to_tex_file.value),
                           'energy_plot2': '{}/energy2.png'.format(
                               SubDir.tex_figs_rel_to_tex_file.value)}
        template_calc_specific = get_calc_specific_latex_template(
            TexTempl.BallOnDisk.value, sim)
        with open(calc_spec_tex_file_handle, 'w') as f:
            f.write(template_calc_specific.render(latex_variables))

    def generate_latex_figures(self, ui=None, res_dir=None):
        """Generate calculation-specific part of the LaTeX output file"""
        plt_profile_approx(res_dir, SubDir.tex_figs.value)
        plt_contact(self.ball, self.disk, PltOpts.DDD.value, res_dir,
                    SubDir.tex_figs.value, 'contact1')
        plt_3d(self.ball.x_axis, self.ball.y_axis, self.ball.press,
               self.ball.x_label, self.ball.y_label,
               'pressure in {}'.format(UnitTex.pressure.value),
               'contact_pressure_ball', res_dir,
               SubDir.tex_figs.value, 'pressure1')
        plt_2d_2y_ax(self.ball.x_axis, self.ball.e_akin, self.ball.x_axis,
                     self.pv, self.ball.x_label,
                     'e_akin vs pv_rel',
                     'e_akin in {}'.format(UnitTex.eakin.value),
                     'pv_rel in {}'.format(UnitTex.pvrel.value), res_dir,
                     SubDir.tex_figs.value, 'energy1')
        plt_2d_2y_ax(self.disk.x_axis, self.disk.e_akin, self.disk.x_axis,
                     self.pv, self.disk.x_label,
                     'e_akin vs pv_rel',
                     'e_akin in {}'.format(UnitTex.eakin.value),
                     'pv_rel in {}'.format(UnitTex.pvrel.value), res_dir,
                     SubDir.tex_figs.value, 'energy2')




import os
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import numpy as np
from Constants import PltOpts, CMaps


def plt_surf(m, n, pixel_width, z):
    """
    function to plot output of artificial surf. only used for debugging
    """
    x = np.linspace(0, (m - 1) * pixel_width, m)
    y = np.linspace(0, (n - 1) * pixel_width, n)
    x_grid, y_grid = np.meshgrid(x, y, sparse=False, indexing='ij')

    fig = plt.figure()
    ax = Axes3D(fig)
    ax.plot_surface(x_grid, y_grid, np.transpose(z),
                    rstride=1,
                    cstride=1,
                    cmap=CMaps.default.value,
                    linewidth=0.5,
                    antialiased=True)
    plt.savefig('{}/test-surf.png'.format(os.getcwd()),
                bbox_inches='tight',
                dpi=PltOpts.res.value)
    plt.close()


def artificial_surf(sigma, h, lx, m, n, qr, eng=None):
    """
    Calls matlab function "artificial_surf.m" developed by Mona Mahboob Kanafi.
    See file "artificial_surf.m" for copyright notice
    
    parameters
    sigma: standard deviation, i.e.root - mean - square roughness Rq(m)
    H: Hurst exponent(roughness exponent), 0 <= H <= 1
    It relates to the fractal dimension of a surface by D = 3 - H.
    Lx: length of topography in x direction (m)
    m: number of pixels in x
    n: number of pixels in y

    example:
    [z , PixelWidth, PSD] = artificial_surf(0.5e-3, 0.8, 0.1, 512 , 512);
    generates surface z with sigma = 0.5 mm, H = 0.8 Hurst exponent
    [i.e. a surface with fractal dimension D = 2.2], Lx = 10 cm [the size of
    topography in x direction], m = n = 512 [results in a square surface,
    i.e. Lx = Ly = 10 cm]. With these parameters simply the Pixel Width of
    the final topography is Lx/(m-1) = 1.96e-4.
    """

    z, pixel_width, qx, qy, cq, q, c = \
        eng.artificial_surf(float(sigma), float(h), float(lx), float(m),
                            float(n), float(qr), nargout=7)
    [n, m] = z.size
    return np.asarray(z)*1000, pixel_width, qx, qy, cq, q, c, n, m


if __name__ == "__main__":
    pass



import math
import os
import re

import numpy as np

from BluPrintTriboSys import TriboSys
from Constants import PltOpts, SubDir, TexTempl, Unit, UnitTex, PrintOpts, \
    PreSol
from cartesian_plot_functions import plt_contact, plt_3d, plt_2d, \
    plt_2d_scatt_line, plt_2d_2y_ax, \
    plt_profile_approx, plt_cum_dist, plt_profiles
from generate_latex_output import get_calc_specific_latex_template
from hertz_equations import hertz_displ
from influ_matrix_management import load_influ_mat, cache_influ_mat
from solve_half_space import solve_half_space, pre_solve_half_space
from system_functions import print_it, to_preci, save_to_matlab


class PinOnDisk(TriboSys):
    # TODO: add class attribute description
    def __init__(self, num_pins, sliding_diameter, pin, disk, global_force,
                 setup_name='pin-on-disk'):
        super().__init__(setup_name, global_force, None)

        self.num_pins = num_pins
        self.sliding_diameter = sliding_diameter  # in mm
        self.raceway_vel = None
        self.press_zone_len = None
        self.effective_rot_velocity = None
        self.effective_omega = None

        self.influ_mat_db_1 = None

        self.rot_vel1 = None
        self.norm_forces = None

        self.pin = pin
        self.pin.press = None
        self.pin.max_press = None
        self.pin.rel_vel = None

        self.disk = disk
        self.disk.press = None
        self.disk.max_press = None

        self.pv = None
        self.e_akin = None

        self.footpr_vel = None
        self.overroll_t_incr = None
        self.overroll_t = None
        self.no_overroll_t = None

        if self.pin.diameter == self.pin.circle_radius:
            self.pin_is_ball = True
        else:
            self.pin_is_ball = False

    def calc_load_distribution(self, ui=None, res_dir=None):
        """Calculate load distribution"""
        print_it("calculating load distribution")
        self.norm_forces = np.multiply(np.ones(self.num_pins),
                                       self.global_force / self.num_pins)
        self.init_force = self.global_force / self.num_pins

    def get_grid_size(self, ui=None, res_dir=None):
        """Determine grid size by running (quick) simulation with simplified
        contact bodies"""
        print_it('determining grid size', PrintOpts.lvl1.value)
        self.pin.simple_clone()
        self.pin.clone.make_profile(PreSol.res_x.value, PreSol.res_y.value,
                                    self.init_force)
        self.disk.simple_clone()
        self.disk.clone.make_slave_to(self.pin.clone)

        init_displ = hertz_displ(self.pin.clone.e, self.disk.clone.e,
                                 self.pin.clone.ny, self.disk.clone.ny,
                                 self.pin.clone.r_hertz_x,
                                 self.pin.clone.r_hertz_y,
                                 self.disk.clone.r_hertz_x,
                                 self.disk.clone.r_hertz_y, self.norm_forces[0])
        contact_delta = 1
        contact_width = 0.025
        while contact_delta != 0:
            if type(self.pin).__name__ == "Ring":
                self.pin.clone.make_profile(self.pin.clone.res_x,
                                            self.pin.clone.res_y,
                                            self.init_force,
                                            contact_width=contact_width)
            elif type(self.pin).__name__ == "Ball":
                self.pin.clone.make_profile(PreSol.res_x.value,
                                            PreSol.res_y.value, self.init_force,
                                            contact_width=contact_width)
                self.disk.make_slave_to(self.pin)

            pressure, init_displ = \
                pre_solve_half_space(self.pin.clone.profile,
                                     self.disk.clone.profile,
                                     self.pin.clone.x_axis,
                                     self.pin.clone.y_axis,
                                     self.pin.clone.res_x, self.pin.clone.res_y,
                                     self.pin.clone.delta_x,
                                     self.pin.clone.delta_y, self.pin.clone.e,
                                     self.disk.clone.e, self.pin.clone.ny,
                                     self.disk.clone.ny, self.norm_forces[0],
                                     init_displ=init_displ)
            pressure_elements = sum(
                pressure[math.floor(self.pin.clone.res_y / 2), :] > 0)
            contact_delta = pressure_elements + 2 - self.pin.clone.res_y
            contact_width += np.sign(contact_delta) * contact_width / 25

        if type(self.pin).__name__ == "Ring":
            self.pin.make_profile(self.pin.res_x, self.pin.res_y,
                                  self.init_force, contact_width=contact_width)
        elif type(self.pin).__name__ == "Ball":
            self.pin.make_profile(self.pin.res_x, self.pin.res_y,
                                  self.init_force, contact_width=contact_width)
            self.disk.make_slave_to(self.pin)
        return init_displ

    def calc_contact_pressure(self, ui=None, res_dir=None):
        """Calculate contact pressure distribution between pin and disk"""
        print_it('calculating 1 pressure distribution')

        init_displ = self.get_grid_size(ui, res_dir)
        [self.influ_mat_db_1] = load_influ_mat(ui, res_dir, 1)
        print_it('solving first half space', PrintOpts.lvl1.value)
        self.pin.press, self.influ_mat_db_1 = \
            solve_half_space(self.pin.profile, self.disk.profile,
                             self.pin.x_axis, self.pin.y_axis, self.pin.res_x,
                             self.pin.res_y, self.pin.delta_x, self.pin.delta_y,
                             self.pin.e, self.disk.e,
                             self.pin.ny, self.disk.ny, self.norm_forces[0],
                             res_dir, init_displ=init_displ,
                             influ_mat_db=self.influ_mat_db_1)

        cache_influ_mat(ui, [self.influ_mat_db_1], res_dir)
        self.pin.max_press = np.amax(self.pin.press, axis=1)
        self.save_pressure_to_matlab(res_dir)

    def save_pressure_to_matlab(self, res_dir):
        dat_dict = dict(contact_pressure=self.pin.press,
                        x_axis=self.pin.x_axis,
                        y_axis=self.pin.y_axis)
        save_to_matlab(dat_dict, res_dir, 'pressure-field')

    def calc_kinematics(self, rot_vel1, rot_vel2, ui=None, res_dir=None):
        """Calculate bearing kinematics based on rotational velocity of disk"""
        print_it("calculating kinematics")
        self.rot_vel1 = rot_vel1
        self.effective_rot_velocity = self.rot_vel1
        self.effective_omega = self.effective_rot_velocity / 60
        self.pin.rel_vel = np.add(-self.pin.x_axis,
                                  self.sliding_diameter) * \
                                        math.pi * self.effective_omega

        self.press_zone_len = (self.pin.press > 0).sum(1) * self.pin.delta_y
        self.footpr_vel = self.pin.rel_vel
        self.overroll_t_incr = self.pin.delta_y / self.footpr_vel
        self.overroll_t = np.divide(self.press_zone_len, self.footpr_vel)
        self.no_overroll_t = np.divide((2 * math.pi * (
        self.sliding_diameter / 2) - self.num_pins * self.press_zone_len)
                                       / self.num_pins, self.footpr_vel)

    def calc_pv(self, ui=None, res_dir=None):
        """Calculate product of local maximum pressure and local maximum
        relative velocity"""
        print_it("calculating pv_rel")
        self.pv = np.multiply(self.pin.max_press, self.pin.rel_vel) / 1000

    def calc_e_akin(self, ui=None, res_dir=None):
        """"Calculate the kinetic friction energy accumulation in W per m^2"""
        print_it("calculating e_a,kin")
        pv_local = np.multiply(self.pin.press.sum(1), self.pin.rel_vel)
        self.e_akin = np.absolute(
            np.divide(np.multiply(pv_local, self.overroll_t_incr),
                      self.no_overroll_t)) / 1000

    def plot_it(self, ui=None, res_dir=None):
        """Orchestrate output plot generation"""
        print_it("plotting results")
        plt_profiles(res_dir)
        print_it("plotting other", PrintOpts.lvl1.value)
        plt_contact(self.pin, self.disk, PltOpts.DD.value, res_dir,
                    SubDir.contacts.value)
        plt_contact(self.pin, self.disk, PltOpts.DDD.value, res_dir,
                    SubDir.contacts.value)
        plt_3d(self.pin.x_axis, self.pin.y_axis, self.pin.press,
               self.pin.x_label, self.pin.y_label,
               'pressure in MPa', 'contact_pressure_pin', res_dir,
               SubDir.pressures.value, 'contact_pressure')
        plt_2d(self.pin.x_axis, self.pin.rel_vel, self.pin.x_label, 'rel vel',
               'rel vel', res_dir, SubDir.energy.value,
               'rel_vel')
        plt_2d(self.pin.x_axis, self.pv, self.pin.x_label,
               'pv_rel in {}'.format(Unit.pvrel.value), 'pv_rel', res_dir,
               SubDir.energy.value, 'pv_rel')
        plt_2d_scatt_line(self.pin.x_axis, self.e_akin, self.pin.x_axis,
                          self.e_akin, self.pin.x_label,
                          'e_akin in W $m^{-2}$', 'e_akin', res_dir,
                          SubDir.energy.value, 'e_akin')
        plt_2d_2y_ax(self.pin.x_axis, self.e_akin, self.pin.x_axis, self.pv,
                     self.pin.x_label, 'e_akin vs pv_rel',
                     'e_akin in {}'.format(Unit.eakin.value),
                     'pv_rel in {}'.format(Unit.eakin.value),
                     res_dir, SubDir.energy.value, 'e_akin_vs_pv_rel')

        plt_cum_dist(self.pin.press, 'pressure in {}'.format(Unit.press.value),
                     res_dir, SubDir.pressures.value,
                     'cumulative pressure distribution')

    def generate_latex_output(self, calc_spec_tex_file_handle, sim, ui=None,
                              res_dir=None):
        """Generate calculation-specific part of the LaTeX output file"""
        average_pressure = np.mean(self.pin.press[self.pin.press > 0])
        numeric_output_data = [
            ('pressure, max.', to_preci(np.amax(self.pin.press), 4),
             UnitTex.pressure.value, 'unverified'),
            ('pressure, av.', to_preci(average_pressure, 4),
             UnitTex.pressure.value, 'unverified'),
            ('e_a,kin, max.', to_preci(np.amax(self.e_akin), 4),
             UnitTex.eakin.value, 'unverified'),
            ('pv_rel, max.', to_preci(np.amax(self.pv), 4),
             UnitTex.pvrel.value, 'unverified'),
            ('contact area', to_preci(self.pin.get_area(self.pin.press), 4),
             UnitTex.area.value, 'unverified')]

        table_calc_summary = []
        for key, value, unit, status in sorted(numeric_output_data):
            table_calc_summary.append(
                (re.sub('_', '\_', key), value, unit, status))

        latex_variables = {'table_calc_summary': table_calc_summary,
                           'contact_plot1': '{}{}contact1.png'.format(
                               SubDir.tex_figs_rel_to_tex_file.value, '/'),
                           'pressure_plot1': '{}{}pressure1.png'.format(
                               SubDir.tex_figs_rel_to_tex_file.value, '/'),
                           'energy_plot1': '{}{}energy1.png'.format(
                               SubDir.tex_figs_rel_to_tex_file.value, '/')}
        template_calc_specific = get_calc_specific_latex_template(
            TexTempl.PinOnDisk.value, sim)
        with open(calc_spec_tex_file_handle, 'w') as f:
            f.write(template_calc_specific.render(latex_variables))

    def generate_latex_figures(self, ui=None, res_dir=None):
        """Generate calculation-specific figures for LaTeX report"""
        plt_contact(self.pin, self.disk, PltOpts.DDD.value, res_dir,
                    SubDir.tex_figs.value, 'contact1')
        plt_profile_approx(res_dir, SubDir.tex_figs.value)
        plt_3d(self.pin.x_axis, self.pin.y_axis, self.pin.press,
               self.pin.x_label, self.pin.y_label, 'pressure in MPa',
               'contact_pressure_pin', res_dir, SubDir.tex_figs.value,
               'pressure1')

        plt_2d_2y_ax(self.pin.x_axis, self.e_akin, self.pin.x_axis, self.pv,
                     self.pin.x_label, 'e_akin vs pv_rel',
                     'e_akin in W $\mathregular{m^{-2}}$',
                     'pv_rel in W $\mathregular{m^{-2}}$', res_dir,
                     SubDir.tex_figs.value, 'energy1')



import os
from enum import Enum
from matplotlib import cm


class SignCheck(Enum):
    """Options for signs of numeric variables"""
    no_check = 0
    pos = 1
    neg = -1


class OddEvenCheck(Enum):
    """Options for odd-even check of numeric variables"""
    Odd = 1
    Even = 0
    Undefined = None


class CMaps(Enum):
    """Colormap options"""
    default = cm.get_cmap('PuBu')
    markers = default
    contacts = default


class VarType(Enum):
    """Variable type options"""
    string = 0
    integer = 1
    real_number = 2
    bool = 3


class PrintOpts(Enum):
    """Print levels used to structure the console output"""
    lvl0 = ""
    lvl1 = "\t"
    lvl2 = "\t\t"


class SimType(Enum):
    """Types of tribosystems that can be simulated using p3can"""
    cyl_rol_bearing = 1
    deep_gro_ball_bearing = 2
    cyl_rol_thrust_bearing = 3
    ball_on_disk = 4
    pin_on_disk = 5
    four_ball = 6
    ball_on_three_plates = 7
    ring_on_ring = 8


class Profiles(Enum):
    """Profile types for contact bodies"""
    ISO = 'ISO'
    Circle = 'Circle'
    File = 'File'
    NoProfile = 'None'


class SubDir(Enum):
    """Subdirectories of the root directory that are used to save calculation
    outputs"""
    contacts = "plots-contact"
    profiles = "plots-profile"
    pressures = "plots-pressure"
    energy = "plots-energy-figures"
    load_distr = "plots-load-distribution"
    matlab = "matlab-files"
    latex_files = "latex-files"
    tex_figs_rel_to_tex_file = "figures"
    tex_figs = "{}{}{}".format(latex_files, os.sep, tex_figs_rel_to_tex_file)
    pub_figs = "publication-figures"
    np_db = "numpy-databases"
    infl_mat_db_folder = 'cache'


class MasterDir(Enum):
    """Names of top level root directories"""
    latex_templates = os.sep.join(os.path.realpath(__file__)
                                  .split(os.sep)[:-1]) \
                      + os.sep + "LatexTemplates"
    results = "Results"


class NpDBs(Enum):
    """Constants related to influence matrix storage and cache size"""
    infl_mat_db = 'infl-mat-db.pickle'  # name of influence matrix database
    max_cache_size = 1024  # maximum size of influence matrix cache in megabyte


class Unit(Enum):
    """Units of physical quantities for python/matplotlib output"""
    press = "MPa"
    force = "N"
    eakin = "W $\mathregular{m^{-2}}$"
    pvrel = "W $\mathregular{m^{-2}}$"
    area = "W $\mathregular{m^{-2}}$"


class UnitTex(Enum):
    """Units of physical quantities for LaTeX output"""
    pressure = "MPa"
    force = "N"
    eakin = "W m$^{-2}$"
    pvrel = "W m$^{-2}$"
    area = "mm$^{2}$"


class TexTempl(Enum):
    """Names of LaTeX templates"""
    CylindricalRollerBearing = "template-cylindrical-roller-bearing.tex"
    CylindricalRollerThrustBearing = "template-cylindrical-roller-" \
                                     "thrust-bearing.tex"
    PinOnDisk = "template-pin-on-disk.tex"
    FourBall = "template-four-ball.tex"
    BallOn3Plates = "template-ball-on-three-plates.tex"
    RingOnRing = "template-ring-on-ring.tex"
    BallOnDisk = "template-ball-on-disk.tex"
    Generic = 'p3can-calculation-report-template.tex'


class PreSol(Enum):
    """Grid resolution used for the pre-solver when determining a grid size for
     the actual half space calculation"""
    res_x = 13
    res_y = 15


class AutoReportOptions(Enum):
    """Options for automatic LaTeX report generation"""
    true = True
    false = False


class PltOpts(Enum):
    """Options for output plots"""
    DD = '2D'
    DDD = '3D'
    res = 96     # default output plot resolution in dpi
    msize = 75  # default marker size in scatter plots
    azim = 120  # default spacial orientation of non-standard 3D plots


class VarDicts(Enum):
    """Required and optional variables for ALL simulation types.
    Simulation-specific required and optional variables
    are handled elsewhere"""
    required_user_input = {'res_x': VarType.integer,
                           'res_y': VarType.integer,
                           'e_cb1': VarType.real_number,
                           'ny_cb1': VarType.real_number,
                           'diameter_cb1': VarType.real_number,
                           'e_cb2': VarType.real_number,
                           'ny_cb2': VarType.real_number,
                           'global_force': VarType.real_number,
                           'rot_velocity1': VarType.real_number,
                           'simulation_name': VarType.string,
                           'simulation_type': VarType.integer}
    optional_user_input = {'tribo_system_name': VarType.string,
                           'auto_print': VarType.bool,
                           'auto_plot': VarType.bool,
                           'auto_report': VarType.bool,
                           'rms_roughness_cb1': VarType.real_number,
                           'rms_roughness_cb2': VarType.real_number}


class VarProps(Enum):
    """Properties of variables that apply independent of simulation type"""
    positive_vars = ['res_x', 'res_y', 'res_z', 'e_cb1', 'ny_cb1',
                     'diameter_cb1', 'length_cb1', 'e_cb2', 'ny_cb2',
                     'diameter_cb2', 'length_cb2', 'e_cb3', 'ny_cb3',
                     'diameter_cb3', 'length_cb3', 'global_force', 'res_pol',
                     'number_rollers', 'number_pins', 'number_planets',
                     'sliding_diameter', 'mean_diameter', 'plate_angle',
                     'rms_roughness_cb1', 'rms_roughness_cb2',
                     'rms_roughness_cb3']
    negative_vars = []
    alphanumeric_vars = ['simulation_name', 'tribo_system_name']
    odd_vars = ['res_x', 'res_y', 'res_z']
    bool_vars = ['auto_print', 'auto_plot', 'auto_report']
    int_vars = ['number_pins', 'number_rollers', 'number_planets', 'res_x',
                'res_y', 'res_z', 'res_pol']



from setuptools import setup

setup(
   name='tribology',
   version='0.2.92',
   setup_requires=['setuptools-git-version'],
   description='functions and classes for tribology research',
   long_description='A collection of functions and classes for tribology research '
                    'and education, including contact mechanics, '
                    'lubrication science and data handling.',
   author='Moritz Ploss',
   author_email='moritz.ploss@gmail.com',
   packages=['tribology',
             'tribology.p3can',
             'tribology.tests'
             ],
   include_package_data=True,
   license='MIT',
   keywords='tribology machine design research',
   install_requires=['numpy', 'numexpr', 'scipy'],
   python_requires='>=3.5',
   url='https://github.com/moritzploss/tribology',
   classifiers=[
    # How mature is this project? Common values are
    #   3 - Alpha
    #   4 - Beta
    #   5 - Production/Stable
    'Development Status :: 4 - Beta',

    # Indicate who your project is intended for
    'Intended Audience :: Science/Research',
    'Topic :: Scientific/Engineering',

    # Pick your license as you wish (should match "license" above)
    'License :: OSI Approved :: MIT License',

    # Specify the Python versions you support here. In particular, ensure
    # that you indicate whether you support Python 2, Python 3 or both.
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6'],
)



import copy
import math
import os
import uuid

import numexpr as ne
import numpy as np
import scipy.sparse.linalg as spla

from Constants import PrintOpts, SubDir
from numeric_methods import secant_method
from system_functions import print_progress, print_it, make_directory, \
    print_rel_prox_to_sol


def solve_for_pressure(profile1, profile2, normal_load, red_influ_mat,
                       init_displ, delta_x, delta_y, print_prog=True):
    """
    Solve system of equations:
    
        [pressure] = [influence matrix]^-1 * [displacement]
        
    Stop solver once inner and outer forces are in equilibrium
    """

    # initialise variables
    x_value = [0]
    fx_value = [normal_load]
    pressure = []
    resulting_force = 0
    profile = -(profile1 + profile2)
    threshold_factor = 0.005

    # do while difference between inner forces and outer forces is significant
    while abs(resulting_force - normal_load) > threshold_factor * normal_load:
        displacement_field = np.subtract(np.ones(profile.shape) * init_displ,
                                         profile)
        u = np.reshape(displacement_field,
                       (profile.shape[0] * profile.shape[1]))

        # lots of options to solve this, the active implementation proved to be
        # the fastest for me
        # p = np.linalg.solve(red_influ_mat, u)
        p = spla.gmres(red_influ_mat, u)[0]

        p_index = np.zeros(len(p))
        negative_p = np.where(p < 0)[0]
        p_neg = copy.deepcopy(negative_p)

        # do while there are elements with negative pressure
        while len(negative_p) > 0:
            p[p_neg] = 0
            p_index[p_neg] = 1
            u_new_reduced = np.delete(u, [p_neg], axis=0)
            g_new_reduced = np.delete(red_influ_mat, [p_neg], axis=0)
            g_new_reduced = np.delete(g_new_reduced, [p_neg], axis=1)
            if p[np.where(p_index == 0)].size > 0:
                # again, lots of options available here, this seems to be the
                # fastest for most cases
                p[np.where(p_index == 0)] = \
                spla.gmres(g_new_reduced, u_new_reduced)[0]
            negative_p = np.where(p < 0)[0]
            p_neg = np.append(p_neg, negative_p)

        # calculate resulting force and adjust displacement for next calculation
        # loop
        pressure = np.reshape(p, (profile.shape[0], profile.shape[1]))
        resulting_force = sum(sum(np.multiply(delta_x * delta_y, pressure)))
        x_value = np.append(x_value, [init_displ])
        fx_value = np.append(fx_value, [resulting_force - normal_load])
        init_displ = secant_method(x_value, fx_value)
        if print_prog is True:
            print_rel_prox_to_sol(resulting_force, normal_load,
                                  threshold_factor)
    return pressure, resulting_force, x_value[-1]


def pre_solve_half_space(profile1, profile2, x_axis, y_axis, res_x, res_y,
                         delta_x, delta_y, e1, e2, ny1, ny2,
                         normal_load, init_displ=0.0005, print_prog=False):
    """Solve half space with simplified geometries to find grid size for actual
    simulation"""
    red_influ_mat = calc_reduced_influence_matrix(x_axis, y_axis, res_x, res_y,
                                                  delta_x, delta_y, e1, e2, ny1,
                                                  ny2,
                                                  print_prog=False)
    pressure, resulting_force, init_displ = solve_for_pressure(profile1,
                                                               profile2,
                                                               normal_load,
                                                               red_influ_mat,
                                                               init_displ,
                                                               delta_x, delta_y,
                                                               print_prog)
    return pressure, init_displ


def solve_half_space(profile1, profile2, x_axis, y_axis, res_x, res_y, delta_x,
                     delta_y, e1, e2, ny1, ny2, normal_load,
                     res_dir, influ_mat_db='empty', print_prog=True,
                     init_displ=0.0005):
    """Orchestrate half space calculation and influence matrix
    generation/retrieval"""
    influ_mat_path = os.sep.join([res_dir, SubDir.np_db.value, influ_mat_db])
    if os.path.exists(influ_mat_path):
        red_influ_mat = np.load(influ_mat_path)
    else:
        print_it('calculating influence matrix', PrintOpts.lvl1.value)
        red_influ_mat = calc_reduced_influence_matrix(x_axis, y_axis, res_x,
                                                      res_y, delta_x, delta_y,
                                                      e1, e2, ny1, ny2,
                                                      print_prog=True)
        print_it('saving influence matrix', PrintOpts.lvl1.value)
        np_db_directory = make_directory(res_dir, SubDir.np_db.value)
        influ_mat_db = 'influence-matrix-{}.npy'.format(uuid.uuid4())
        np.save(os.sep.join([np_db_directory, influ_mat_db]), red_influ_mat)
        print_it('solving first half space', PrintOpts.lvl1.value)

    pressure, resulting_force, init_displ = \
        solve_for_pressure(profile1, profile2, normal_load, red_influ_mat,
                           init_displ, delta_x, delta_y, print_prog)
    return pressure, influ_mat_db


def calc_reduced_influence_matrix(x_axis, y_axis, res_x, res_y, delta_x,
                                  delta_y, e1, e2, ny1, ny2, print_prog=False):
    """Calculate the complete influence matrix C and extract a reduced influence
    matrix"""

    # initialise variables
    influence_matrix_complete = np.zeros((res_x, res_y, res_x, res_y))
    reduced_influence_matrix = np.zeros((res_x * res_y, res_x * res_y))
    youngs_mod_over_pi = ((1 - math.pow(ny1, 2)) / e1 + (
        1 - math.pow(ny2, 2)) / e2) / math.pi
    a = delta_x / 2
    b = delta_y / 2

    # generate x coordinate grid
    x_new = np.zeros((1, res_x))
    x_new[0, :] = x_axis
    vertical_x_axis = np.zeros((res_x, 1))
    vertical_x_axis[:, 0] = x_axis
    x = np.repeat(x_new, res_x, axis=0)
    x = np.absolute(np.subtract(x, vertical_x_axis))

    # generate y coordinate grid
    y_new = np.zeros((1, res_y))
    y_new[0, :] = y_axis
    vertical_y_axis = np.zeros((res_y, 1))
    vertical_y_axis[:, 0] = y_axis
    y = np.repeat(y_new, res_y, axis=0)
    y = np.absolute(np.subtract(y, vertical_y_axis))

    # use numexpr to evaluate expressions
    xpa = ne.evaluate('x + a')
    xma = ne.evaluate('x - a')
    ypb = ne.evaluate('y + b')
    ymb = ne.evaluate('y - b')

    '''
    The code in comments below is a simple implementation to calculate the 
    reduced influence matrix. Due to its 4 nested for-loops, it's quite slow, 
    but it's readable. The active implementation (see further below) is 
    basically the same code, but in vectorized from, which makes it much faster, 
    but less readable. It is possible to vectorize the active implementation 
    even further and to get rid of another two for-loops; however, this leads 
    to matrix operations on very large matrices, and my notebook had problems 
    with that at larger grid sizes. also, for high grid resolution
    the current bottleneck is solving the half space, not generating the 
    influence matrix.
        -- Moritz
    
    influence_matrix = np.zeros((res_x, res_y))
    counter = 0
    for i in range(0, res_x):
        print_progress(i, res_x)
        for j in range(0, res_y):
            for i_prime in range(0, res_x):
                x = abs(x_axis[i] - x_axis[i_prime])
                for j_prime in range(0, res_y):
                    y = abs(y_axis[j] - y_axis[j_prime])

                    influence_matrix[i_prime, j_prime] = youngs_mod_over_pi * \
                            (
                        (x + a) * math.log(((y + b) + math.sqrt(math.pow((y + b), 2) + math.pow((x + a), 2))) / (
                            (y - b) + math.sqrt(math.pow((y - b), 2) + math.pow((x + a), 2)))) +
                        (y + b) * math.log(((x + a) + math.sqrt(math.pow((y + b), 2) + math.pow((x + a), 2))) / (
                            (x - a) + math.sqrt(math.pow((y + b), 2) + math.pow((x - a), 2)))) +
                        (x - a) * math.log(((y - b) + math.sqrt(math.pow((y - b), 2) + math.pow((x - a), 2))) / (
                            (y + b) + math.sqrt(math.pow((y + b), 2) + math.pow((x - a), 2)))) +
                        (y - b) * math.log(((x - a) + math.sqrt(math.pow((y - b), 2) + math.pow((x - a), 2))) / (
                            (x + a) + math.sqrt(math.pow((y - b), 2) + math.pow((x + a), 2))))
                            )

                    #influence_matrix[i_prime, j_prime] = ne.evaluate(eval_string)
            reduced_influence_matrix[counter, :] = np.reshape(influence_matrix, (res_x * res_y))
            counter += 1
    '''

    # this code is a vectorized version of the above code. the code assembles
    # the complete influence matrix
    for j in range(0, res_y):
        if print_prog is True:
            print_progress(j, res_y)
        for j_prime in range(0, res_y):
            influence_matrix_complete[:, j, :, j_prime] = \
                (
                    np.multiply(xpa, np.log(
                        np.divide(
                            ((ypb[j, j_prime]) + np.sqrt(
                                np.multiply((ypb[j, j_prime]),
                                            (ypb[j, j_prime])) +
                                np.multiply(xpa, xpa)))
                            ,
                            ((ymb[j, j_prime]) + np.sqrt(
                                np.multiply((ymb[j, j_prime]),
                                            (ymb[j, j_prime])) +
                                np.multiply(xpa, xpa)))
                        )
                    ))
                    +
                    (ypb[j, j_prime]) * np.log(
                        np.divide(
                            (xpa + np.sqrt(np.multiply((ypb[j, j_prime]),
                                                       (ypb[j, j_prime])) +
                                           np.multiply(xpa, xpa)))
                            ,
                            (xma + np.sqrt(np.multiply((ypb[j, j_prime]),
                                                       (ypb[j, j_prime])) +
                                           np.multiply(xma, xma)))
                        )
                    )
                    +
                    np.multiply(xma, np.log(
                        np.divide(
                            ((ymb[j, j_prime]) + np.sqrt(
                                np.multiply((ymb[j, j_prime]),
                                            (ymb[j, j_prime])) +
                                np.multiply(xma, xma)))
                            ,
                            ((ypb[j, j_prime]) + np.sqrt(
                                np.multiply((ypb[j, j_prime]),
                                            (ypb[j, j_prime])) +
                                np.multiply(xma, xma)))
                        )
                    ))
                    +
                    (ymb[j, j_prime]) * np.log(
                        np.divide(
                            (xma + np.sqrt(np.multiply((ymb[j, j_prime]),
                                                       (ymb[j, j_prime])) +
                                           np.multiply(xma, xma)))
                            ,
                            (xpa + np.sqrt(np.multiply((ymb[j, j_prime]),
                                                       (ymb[j, j_prime])) +
                                           np.multiply(xpa, xpa)))
                        )
                    )
                )

    influence_matrix_complete *= youngs_mod_over_pi

    # extract the reduced influence matrix from the complete influence matrix
    counter = 0
    for i in range(0, res_x):
        for j in range(0, res_y):
            reduced_influence_matrix[counter, :] = np.reshape(
                influence_matrix_complete[i, j, :, :], (res_x * res_y))
            counter += 1

    return reduced_influence_matrix



# -*- coding: utf-8 -*-

"""

The below functions can be used to import delimited data files into Numpy or
Matlab database format.

"""

import argparse
import glob
import os
import re
import copy
import math

import numpy as np
# pylint: disable=no-member
import scipy.io


class _Colors:
    """

    A collection of colors that can be used to highlight terminal outputs.

    """

    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


def __print_status(message, status_color=_Colors.ENDC):
    """

    Print a color-coded message to the terminal.

    Parameters
    ----------
    message: str
        The message to print to the terminal.
    status_color:
        The color in which to print the message.

    """
    print(status_color + message + _Colors.ENDC)


def __is_floatable(num):
    """

    Check if 'num' can be converted to float. If yes, return :code:`True`, else
    return :code:`False`.

    """
    try:
        float(num)
        return True
    except ValueError:
        return False


def __to_float(num):
    """

    Try to convert 'num' to float, return 'num' if it's not possible, else
    return converted :code:`num`.

    """
    try:
        float(num)
        return float(num)
    except ValueError:
        return num


def __assemble_data_table(num_data_tables, max_num_data_length):
    """

    Assemble the complete data table from a list of data tables.

    """
    num_data = np.zeros((
        (len(num_data_tables) - 1) * max_num_data_length +
        num_data_tables[-1].shape[0],
        num_data_tables[-1].shape[1]), dtype=object)
    for idx, data_table in enumerate(num_data_tables):
        # do this for all but the last data table
        if idx + 1 < len(num_data_tables):
            num_data[idx * max_num_data_length:
                     (idx + 1) * max_num_data_length, :] = data_table
        # do this for the last data table
        else:
            num_data[idx * max_num_data_length:, :] = data_table
    return num_data


def __write_to_out_dict(num_data, column_headers, pcs=False):
    """

    Extract the data columns from the num_data array and write them to a
    dictionary.

    Parameters
    ----------
    num_data: ndarray
        The data extracted from the delimited file, stored in a single table.
    column_headers: ndarray
        The column headers corresponding to the columns in :code:`num_data`

    Returns
    -------
    out_dict: dict
        A dictionary containing all data that is to be saved to the output
        database. Keys are based on column headers, values are data columns of
        num_data.

    """
    out_dict = {'column_headers': column_headers}
    for idx, column in enumerate(column_headers):
        # explicitly take care of the fact that PCS forgot a '\tab' character in
        # their data export implementation
        if column == 'image_file_name' and \
                math.isnan(float(num_data[0, idx])) and not \
                column_headers[column_headers.tolist().index(column) - 1] and \
                pcs is True:
            out_dict[column] = num_data[:, idx - 1].astype(object)[:, None]
        # take care of all other columns
        # if empty data columns are not padded with tabs
        elif column:
            if idx >= num_data.shape[1]:
                out_dict[column] = np.zeros(num_data.shape[1]) * float('nan')
            else:
                # if data is of numeric type
                if __is_floatable(num_data[0, idx]):
                    out_dict[column] = num_data[:, idx].astype(float)[:, None]
                # if data is of other type (string)
                else:
                    out_dict[column] = num_data[:, idx].astype(object)[:, None]
    return out_dict


def __process_header(prev_line):
    """

    Process the column headers by removing special characters and converting to
    Matlab-optimized data type.

    Parameters
    ----------
    prev_line: list of strings
        The column headers of the delimited file.

    Returns
    -------
    col_heads: ndarray (dtype = object)
        The re-formated column headers.

    """

    # replace non-alphanumeric characters and trailing underscores
    col_heads = [re.sub("\W+", '_', item.lower()).strip('_')
                 for item in prev_line]
    # convert data type for easy matlab export
    col_heads = np.asarray(col_heads, dtype='object')
    return col_heads


def __process_data(split_line, num_dat, max_len, num_data_tables):
    """

    Append a data line to the current data table. If the length of the current
    data table exceeds the maximum permitted data table length, save the current
    data table to a list of data tables and initialise a new one.

    Parameters
    ----------
    split_line: ls
        The data that is to be appended to the table.
    num_dat: ndarray
        The current data table to which the last line of data was appended.
    max_len: positive int
        The maximum length of a data table.
    num_data_tables: ls
        The complete list of data tables.

    Returns
    -------
    num_dat: ndarray
        The data table to which the current line of data was appended.

    """
    # if data table becomes large, make new data table and add old
    # table to table list (for speed)
    if num_dat.shape[0] == max_len:
        num_data_tables.append(num_dat)
        num_dat = np.asarray(
            [__to_float(item.rstrip('\n')) for item in
             split_line]).reshape((1, len(split_line)))
    # else simply append to data table
    else:
        num_dat = np.append(num_dat, np.asarray(
            [__to_float(item.rstrip('\n')) for item in split_line])
                            .reshape((1, len(split_line))), axis=0)
    return num_dat


def __process_file(in_file, dec_mark, deli, pad=0):
    """

    Extract data from a delimited text file and return a dictionary containing
    all data.

    Parameters
    ----------
    in_file: str
        The file handle of the delimited file that is to be imported.
    dec_mark: str
        The decimal mark of the data file.
    deli: str
        The delimiter used to separate data columns in the delimited file.
    pad: positive int
        Ignore the first :code:`n` leading columns in the delimited file, where
        :code:`n = pad`. For example, if pad = 8, the first 8 columns
        are ignored.

    Returns
    -------
    out_dict: dict
        A dictionary containing all data that is to be saved to the output
        database. Keys are based on column headers, values are data columns of
        num_data.

    """
    max_len = 1000
    num_dat = []
    col_heads = []
    num_data_tables = []
    prev_line = ''

    with open(in_file) as dat_file:
        for line in dat_file:
            split_line = line.replace(dec_mark, '.').split(deli)

            if len(split_line) > pad:
                split_line = split_line[pad:]

            # get rid of trailing newline characters
            if split_line[-1] == '\n':
                split_line[-1] = ''

            # check if first character is not (digit or minus symbol (hyphen))
            # to identify non-data lines. skip non-data lines.
            if not (line[0].isdigit() or line[0] == '-') or \
                    len(split_line) <= 1:
                if split_line != ['']:
                    prev_line = split_line
                continue

            # if line contains data, split line into data fields, fill empty
            # fields with 'nan'
            split_line[:] = (item or 'nan' for item in split_line)
            # if this is the first data-containing line...
            if not len(col_heads):
                # get the column headers
                col_heads = __process_header(prev_line)
                # write the first line to the data table
                num_dat = np.asarray(
                    [__to_float(item.rstrip('\n'))
                     for item in split_line]).reshape((1, len(split_line)))
            else:
                num_dat = __process_data(split_line, num_dat, max_len,
                                         num_data_tables)

    # assemble the complete data table and create output dictionary
    num_data_tables.append(num_dat)
    num_dat = __assemble_data_table(num_data_tables, max_len)

    return num_dat, col_heads


def __get_file_handles(in_dir, ext, recursive=False):
    """

    Get file handles for all delimited files that are to be imported.

    Parameters
    ----------
    in_dir: str
        The directory in which the delimited files are stored.
    ext: str
        The file extension of the delimited files.
    recursive: bool, optional
        If :code:`True`, delimited files are imported for all child directories
        of :code:`directory` (including :code:`directory`). If :code:`False`,
        only files in :code:`directory` are imported. Default is :code:`False`.

    Returns
    -------
    in_files: ls of strings
        The file handles to all delimited files that are to be imported.

    """
    if not recursive:
        in_files = sorted(glob.glob('{}{}*.{}'.format(in_dir, os.sep, ext)))
    else:
        in_files = []
        dir_list = [x[0] + os.sep for x in os.walk(in_dir)]
        for directory in dir_list:
            in_files.extend(sorted(glob.glob('{}*.{}'.format(directory, ext))))
        # in_files = [f.replace(in_dir, '').lstrip(os.sep) for f in in_files]
    return in_files


def __save_out_file(out_file, out_dict, out_ext):
    """

    Save the imported data to an output database, either in Numpy or Matlab
    format.

    Parameters
    ----------
    out_file: str
        A handle to the output file that was generated during import.
    out_dict: dict
        The output data stored in a dictionary where keys correspond to column
        headers, values correspond to data.
    out_ext: str
        The file extension (format) of the output file. Options are :code:`npz`
        for Numpy format and :code:`mat` for Matlab database format.

    Returns
    -------
    out_file: str
        A handle to the output file that was generated after import.

    """
    if out_ext == 'mat':
        out_file = '{}.mat'.format(out_file)
        scipy.io.savemat(out_file, out_dict)
    elif out_ext == 'npz':
        out_file = '{}.npz'.format(out_file)
        np.savez(out_file, **out_dict)
    return out_file


def __get_out_file(in_file, out_dir):
    if out_dir == '':
        out_dir = os.path.dirname(in_file)
    file_no_ext = os.path.splitext(in_file)[0].split(os.sep)[-1]
    out_file = '{}/{}'.format(out_dir, file_no_ext)
    return file_no_ext, out_dir, out_file


def __import_file(in_file, out_file, out_ext, force=False, deli='\t',
                  dec_mark='.', pad=0):

    import_status = None
    num_dat = None
    col_heads = None
    out_file_exists = os.path.isfile('{}.{}'.format(out_file, out_ext))

    if (not out_file_exists) or (force is True):
        try:
            num_dat, col_heads = __process_file(in_file, dec_mark, deli,
                                                pad=pad)
            import_status = True
        except (ValueError, AttributeError):
            import_status = False

    return num_dat, col_heads, import_status


def import_del(in_file, force=False, deli='\t', dec_mark='.', out_ext='npz',
               out_dir='', pad=0):
    """

    Import a delimited data file into Numpy or Matlab database format. The file
    must have at least two data columns that are separated by :code:`deli`.

    Parameters
    ----------
    in_file: str
        The file handle of the delimited file that is to be imported.
    force: bool, optional
        If :code:`True`, existing output files will be overwritten during
        import. Default is :code:`False`.
    deli: str, optional
        The delimiter used to separate data columns in the delimited file.
        Default is tab.
    dec_mark: str, optional
        The decimal mark of the data file. Default is dot.
    out_ext: str, optional
        The file extension (format) of the output file. Default is :code:`npz`
        for Numpy database format. Alternative is :code:`mat` for Matlab
        database format.
    out_dir: str, optional
        The absolute or relative path to the output directory. Default is the
        current working directory.
    pad: positive int
        The numbers of data columns to skip. For :code:`pad = n`, the first
        :code:`n` data columns will not be imported.

    Returns
    -------
    out_file: str
        A handle to the output file that was generated during import.
    import_status: str
        The import status of :code:`in_file`. If :code:`True`, the file was
        successfully imported. If :code:`False`, file import was attempted and
        failed. If :code:`None`, file import was not attempted (most likely
        because an output file with the same name already exists).
    out_dict: dict
        The data that was imported from :code:`in_file`.

    """
    _, out_dir, out_file_no_ext = __get_out_file(in_file, out_dir)
    out_dict = None

    num_dat, col_heads, import_status = \
        __import_file(in_file, out_file_no_ext, out_ext, force=force, deli=deli,
                      dec_mark=dec_mark, pad=pad)

    if import_status is True:
        out_dict = __write_to_out_dict(num_dat, col_heads)
        out_file = __save_out_file(out_file_no_ext, out_dict, out_ext)
    else:
        out_file = None

    return out_file, import_status, out_dict


def __gen_acc_time(step_time, outformat='npz'):
    """

    For files produced by PCS Instrument test rigs, generate a continuous time
    axis by combining all step times from all steps.

    """
    # get index of last data point of each step
    current_step_end = np.where(np.subtract(step_time[1:], step_time[0:-1]) < 0)
    step_end = np.append(current_step_end[0], [step_time.shape[0] - 1])

    # get index of first data point of each step
    step_start = np.append([0], [step_end[0:-1] + 1])

    # loop over steps and create continuous time axis
    time_accumulated_s = copy.copy(step_time)
    offset = 0

    for step in range(1, len(step_end)):
        offset += step_time[step_end[step - 1]]
        time_accumulated_s[step_start[step]:step_end[step] + 1] += offset

    # save data to dictionary
    if outformat == 'mat':
        sub_dict = {'time_accumulated_s': time_accumulated_s,
                    'step_start': step_start + 1,
                    'step_end': step_end + 1}
    else:
        sub_dict = {'time_accumulated_s': time_accumulated_s,
                    'step_start': step_start,
                    'step_end': step_end}
    return sub_dict


def __post_process_image_data(out_dict):
    """

    For SLIM Mapper Analysis files produced by PCS Instrument test rigs,
    extract the (x, y) coordinate system, generate an (x, y) grid and map the
    film thickness data to the grid.

    """
    img_dat = {}

    # get (unique) x and y axis values and allocate film thickness matrix
    x_ax = out_dict['x']
    y_ax = out_dict['y']
    x_uniq = np.unique(x_ax)
    y_uniq = np.unique(y_ax)
    x_index = np.zeros(len(x_ax))
    y_index = np.zeros(len(y_ax))
    film = np.zeros((len(x_uniq), len(y_uniq))) * float('nan')

    # get unique rank index for each element in x and y
    for idx, rank_value in enumerate(sorted(x_uniq)):
        x_index[np.where(x_ax == rank_value)[0]] = idx
    for idx, rank_value in enumerate(sorted(y_uniq)):
        y_index[np.where(y_ax == rank_value)[0]] = idx

    # combine x and y indices in a list that can be used to index the film array
    arr_idx = [x_index.astype(int), y_index.astype(int)]

    # assign all measured film thickness values to film thickness matrix
    film[arr_idx] = out_dict['film'][:, 0]

    # create variables that simplify plotting of film thickness data
    img_dat['film_surf'] = film
    img_dat['x_set'] = np.asarray(list(x_uniq))[:, None]
    img_dat['y_set'] = np.asarray(list(y_uniq))[:, None]
    img_dat['x_grid'], img_dat['y_grid'] = \
        np.meshgrid(img_dat['x_set'], img_dat['y_set'], indexing='ij')

    return img_dat


def import_pcs(in_file, force=False, out_ext='npz', out_dir=''):
    """

    Import a delimited data file that was produced by an MTM or EHD2 test rig
    manufactured by PCS Instruments. The method calls the :code:`import_del`
    method to perform a basic import of a delimited text file, and generates
    additional output variables that simplify data analysis.

    Parameters
    ----------
    in_file: str
        The file handle of the delimited file that is to be imported.
    force: bool, optional
        If :code:`True`, existing output files will be overwritten during
        import. Default is :code:`False`.
    out_ext: str, optional
        The file extension (format) of the output file. Default is :code:`npz`
        for Numpy database format. Alternative is :code:`mat` for Matlab
        database format.
    out_dir: str, optional
        The absolute or relative path to the output directory. Default is the
        current working directory.

    Returns
    -------
    out_file: str
        A handle to the output file that was generated during import.
    import_status: str
        The import status of :code:`in_file`. If :code:`True`, the file was
        successfully imported. If :code:`False`, file import was attempted and
        failed. If :code:`None`, file import was not attempted (most likely
        because an output file with the same name already exists).
    out_dict: dict
        The data that was imported from :code:`in_file`.

    """
    _, out_dir, out_file_no_ext = __get_out_file(in_file, out_dir)
    out_dict = None
    out_file = None

    num_dat, col_heads, import_status = \
        __import_file(in_file, out_file_no_ext, out_ext, force=force, deli='\t',
                      dec_mark='.', pad=8)

    if import_status is True:
        out_dict = __write_to_out_dict(num_dat, col_heads, pcs=True)

        if 'step_time_s' in out_dict:
            t_dict = \
                __gen_acc_time(out_dict['step_time_s'].astype(float), out_ext)
            out_dict = {**out_dict, **t_dict}

        try:
            out_dict = {**out_dict, **__post_process_image_data(out_dict)}
        except KeyError:
            pass

        out_file = __save_out_file(out_file_no_ext, out_dict, out_ext)

    return out_file, import_status, out_dict


def __print_import_stats(in_file, status):
    """

    Print the import status to the console.

    Parameters
    ----------
    in_file: str
        The file name of the file for which to print the status.
    status: bool or None
        The import status of :code:`in_file`.

    """
    if status is False:
        out_col = _Colors.FAIL
    elif status is True:
        out_col = _Colors.OKGREEN
    else:
        out_col = _Colors.WARNING

    out_str = '\t'.join([str(status), str(in_file)])
    __print_status(out_str, out_col)


def __parse_args():
    """

    Parse all parser arguments that are provided when the script is running in
    a terminal.

    Returns
    -------
    args: Namespace
        The parsed parser arguments.

    """
    parser = argparse.ArgumentParser()
    parser.add_argument('-f', '--force', action="store_true", default=False,
                        help='overwrite existing database files during import')
    parser.add_argument('-e', '--extension', action="store", default='txt',
                        help='specify file extension. default is "txt"')
    parser.add_argument('-d', '--delimiter', action="store", default='\t',
                        help='specify column delimiter. default is tab (\\t)')
    parser.add_argument('-m', '--mark', action="store", default='.',
                        help='specify decimal mark for numeric data. default is'
                             ' dot (.)')
    parser.add_argument('-o', '--outformat', action="store", default='npz',
                        help='specify output database format. default is "npz"'
                             ' for numpy database. use "mat" for matlab '
                             ' database format.')
    parser.add_argument('-r', '--recursive', action="store_true", default=False,
                        help='recursively walk through all sub-directories of'
                             ' current working directory')
    parser.add_argument('-p', '--pcs', action="store_true", default=False,
                        help='indicate if files are pcs files.')
    args = parser.parse_args()
    return args


def import_dir(in_dir, in_ext='txt', recursive=False, force=False, deli='\t',
               dec_mark='.', out_ext='npz', out_dir='', print_stat=False,
               pcs=False):
    """

    Import all delimited data files in a directory into Numpy or Matlab
    database format. Optionally, all data files in a directory and all its
    child directories can be imported. The method can be applied to regular
    delimited files as well as files generated by test rigs made by PCS
    Instruments. All files must have at least two data columns that are
    separated by :code:`deli`.

    Parameters
    ----------
    in_dir: str
        Path to directory for which to import all files with extension
        :code:`in_ext`. If :code:`recursive=True`, imports are performed for all
        files with extension :code:`in_ext` in the directory tree with parent
        :code:`in_dir`.
    in_ext: str, optional
        File extension of files to import (without dot). Default is :code:`txt`.
    recursive: bool, optional
        If :code:`True`, all files in :code:`in_dir` and all its child
        directories are imported. Default is :code:`False`.
    force: bool, optional
        If :code:`True`, existing output files will be overwritten during
        import. Default is :code:`False`.
    deli: str, optional
        The delimiter used to separate data columns in the delimited file.
        Default is tab.
    dec_mark: str, optional
        The decimal mark of the data file. Default is dot.
    out_ext: str, optional
        The file extension (format) of the output file. Default is :code:`npz`
        for Numpy database format. Alternative is :code:`mat` for Matlab
        database format.
    out_dir: str, optional
        The path to the output directory where output databases are stored after
        import. By default, files are stored in :code:`in_dir` if
        :code:`recursive=False`. If :code:`recursive=True`, files are stored in
        the respective child directories of :code:`in_dir` if :code:`out_dir`
        is not specified.
    print_stat: bool, optional
        If :code:`True`, the current import status is printed to the console.
        Default is :code:`False`.
    pcs: bool, optional
        If :code:`True`, the delimited files are treated like files that were
        generated using an MTM or EHD2 test rig manufactured by PCS Instruments.

    Returns
    -------
    in_files: ls of strings
        The file handles of all files for which import was attempted.
    out_files: ls of strings
        The file handles of all output files that were generated during the
        import process.
    import_status: ls of bools
        The import status of each file in :code:`in_files`. If :code:`True`,
        the file was successfully imported. If :code:`False`, file import was
        attempted and failed. If :code:`None`, file import was not attempted
        (most likely because an output file with the same name already exists).

    """
    in_files = __get_file_handles(in_dir, in_ext, recursive)  # type: ls
    out_files = []
    import_status = []

    if print_stat:
        print('importing {} files'.format(len(in_files)))
        print('status\tfilename\n'
              '======\t========')

    for in_file in in_files:

        if pcs is False:
            out_file, status, _ = import_del(in_file, force=force, deli=deli,
                                             dec_mark=dec_mark, out_ext=out_ext,
                                             out_dir=out_dir)
        else:
            out_file, status, _ = import_pcs(in_file, force=force,
                                             out_ext=out_ext,
                                             out_dir=out_dir)
        out_files.append(out_file)
        import_status.append(status)

        if print_stat:
            __print_import_stats(in_file, status)

    return in_files, out_files, import_status


if __name__ == "__main__":
    # if the file is executed as a script, import all data files in the
    # current working directory based on the parser arguments provided.
    ARGS = __parse_args()
    import_dir(os.getcwd(), in_ext=ARGS.extension, recursive=ARGS.recursive,
               force=ARGS.force, deli=ARGS.delimiter, dec_mark=ARGS.mark,
               out_ext=ARGS.outformat, out_dir=os.getcwd(), print_stat=True,
               pcs=ARGS.pcs)



"""

Test cases for tribology functions

"""
import glob
import unittest
import os
import math

import numpy as np

import sys
sys.path.insert(0, "tribology/p3can")

from ..tribology import profball, profrevolve
from .. import hertz as th
from .. import lubrication as tl
from .. import boundary_element as tb
from .. import data_import as td
from .. import roller_bearings as trb
from ..p3can.p3can import p3can


class TestTribology(unittest.TestCase):
    """
    test case methods for general tribology functions
    """

    def test_dyn2kin(self):
        """
        base test for dyn2kin method
        """
        self.assertEqual(tl.dyn2kin(1, 0.5), 0.5)

    def test_kin2dyn(self):
        """
        base test for kin2dyn method
        """
        self.assertEqual(tl.kin2dyn(1, 0.5), 2)

    def test_effective_modulus(self):
        """
        base test for meff method using steel data
        """
        e_eff = th.eeff(210000, 0.3, 210000, 0.3)
        self.assertEqual(round(e_eff), 230769)

    def test_effective_radii_basic(self):
        """
        base test for eeff method
        """
        r_eff, r_eff_x, r_eff_y = th.reff(6, 0, float('inf'), 3)
        self.assertEqual([r_eff, r_eff_x, r_eff_y], [2, 6, 3])


class TestBoundaryElement(unittest.TestCase):
    """
    test case methods for tribology functions related to boundary element codes
    """
    def test_be(self):
        """
        base test for combination of boundary element functions
        """
        # inputs for steel ball geometry in contact with steel flat
        r_ball = 6.35
        f_outer = 10
        e_eff = th.eeff(210000, 0.3, 210000, 0.3)
        r_eff, r_eff_x, _ = th.reff(r_ball, r_ball, 0, 0)

        # create 3d profile for ball
        width, _, _ = th.ahertz(r_eff, r_eff_x, r_eff_x, e_eff, f_outer)
        ax_x, delta_x = np.linspace(-width * 1.1, width * 1.1, 51, retstep=True)
        prof_ball_3d, _ = profrevolve(profball(ax_x, r_ball), ax_x, 2 * r_ball)

        # calculate influence matrix and reduce it
        inf_mat_red = tb.beinflumatred(
            tb.beinflumat(ax_x, ax_x, e_eff))

        # solve for pressure and inner force in ball <-> flat contact
        press, _, f_inner, _ = tb.besolve(prof_ball_3d,
                                          np.zeros((len(ax_x), len(ax_x))),
                                          f_outer, inf_mat_red, delta_x,
                                          delta_x)

        # verify that boundary element solution is equal to analytical solution
        p_max_hertz = 1.5 * th.phertz(r_eff, r_eff_x, r_eff_x, e_eff, f_inner)
        print(p_max_hertz, np.amax(press))
        self.assertEqual(round(np.amax(press)), round(p_max_hertz))


class TestDowsonHamrock(unittest.TestCase):
    """
    test case methods for tribology functions related to Dowson-Hamrock
    """
    pass


class TestLubrication(unittest.TestCase):
    """
    test case methods for tribology functions related to Lubrication
    """
    pass


class TestRollerBearings(unittest.TestCase):
    """
    test case methods for functions related to roller_bearings
    """

    def test_fcylrolbear(self):
        """
        calculate load distribution in radial roller bearing and compare result
        to reference result
        """
        f_rols, _, _ = trb.fcylrolbear(np.linspace(0, 2 * math.pi, 13),
                                       np.zeros(31),
                                       np.linspace(-5, 5, 31),
                                       3000,
                                       rad_clear=0.0184,
                                       max_dif=0.0005)
        self.assertEqual(round(np.amax(f_rols)), round(967.7))


class TestDataImport(unittest.TestCase):
    """
    test case methods for functions related to data_import
    """
    demo_1 = 'tribology/tests/data_import/demo_1'
    demo_2 = 'tribology/tests/data_import/demo_2'
    demo_1_pcs = 'tribology/tests/data_import/test_data_pcs/demo_1'
    demo_2_pcs = 'tribology/tests/data_import/test_data_pcs/demo_2'

    demo_1_txt = '{}.txt'.format(demo_1)
    demo_2_txt = '{}.txt'.format(demo_2)
    demo_1_pcs_txt = '{}.txt'.format(demo_1_pcs)
    demo_2_pcs_txt = '{}.txt'.format(demo_2_pcs)

    demo_1_npz = '{}.npz'.format(demo_1)
    demo_2_npz = '{}.npz'.format(demo_2)
    demo_1_pcs_npz = '{}.npz'.format(demo_1_pcs)
    demo_2_pcs_npz = '{}.npz'.format(demo_2_pcs)

    demo_dir = 'tribology/tests/data_import'
    demo_dir_pcs = 'tribology/tests/data_import/test_data_pcs'
    demo_dir_rec = 'tribology/tests/data_import/test_data_recursive'

    def test_import_txt_to_npz(self):
        """
        check if simple txt file can be imported to npz format
        """
        f_out, status, _ = td.import_del(self.demo_1_txt)
        self.assertEqual(status, True)
        os.remove(f_out)

    def test_import_txt_database(self):
        """
        check if database is created correctly
        """
        f_out, _, _ = td.import_del(self.demo_1_txt)
        database = np.load(f_out)
        self.assertEqual(database['fx_n'][0], 0.211219)
        os.remove(f_out)

    def test_import_txt_to_mat(self):
        """
        check if simple txt file can be imported to mat format
        """
        f_out, status, _ = td.import_del(self.demo_1_txt, out_ext='mat')
        self.assertEqual(status, True)
        os.remove(f_out)

    def test_import_dir_txt_to_npz(self):
        """
        check if directory containing delimited txt files can be imported to npz
        format
        """
        f_in, f_out, status = td.import_dir(self.demo_dir)
        self.assertEqual(f_in, [self.demo_1_txt, self.demo_2_txt])
        self.assertEqual(f_out[0].endswith(self.demo_1_npz), True)
        self.assertEqual(f_out[1].endswith(self.demo_2_npz), True)
        self.assertEqual(status, [True, True])
        for file in f_out:
            os.remove(file)

    def test_import_pcs_mtm_to_npz(self):
        """
        check if MTM (PCS Instruments) output file can be imported to npz
        """
        f_out, status, _ = td.import_pcs(self.demo_2_pcs_txt, out_ext='npz')
        self.assertEqual(status, True)
        database = np.load(f_out)
        self.assertEqual(database['step_time_s'][0], 23)
        os.remove(f_out)

    def test_import_pcs_ehd_to_npz(self):
        """
        check if EHD2 (PCS Instruments) output file can be imported to npz
        """
        f_out, status, _ = td.import_pcs(self.demo_1_pcs_txt, out_ext='npz')
        self.assertEqual(status, True)
        database = np.load(f_out)
        self.assertEqual(database['film'][0], 130)
        os.remove(f_out)

    def test_import_dir_pcs_to_npz(self):
        """
        check if directory containing pcs files can be imported to npz format
        """
        _, f_out, status = td.import_dir(self.demo_dir_pcs, pcs=True)
        self.assertEqual(status, [True for _ in f_out])
        database = np.load(self.demo_1_pcs_npz)
        self.assertEqual('film_surf' in database, True)
        for file in f_out:
            os.remove(file)

    def test_import_dir_recursive(self):
        """
        check if directory tree can be imported recursively and output files are
        saved together with input files
        """
        _, f_out, status = td.import_dir(self.demo_dir_rec, recursive=True)
        self.assertEqual(status, [True for _ in f_out])
        self.assertEqual(len(f_out), 4)
        for file in f_out:
            os.remove(file)

    def test_import_dir_recursive_out(self):
        """
        check if directory tree can be imported recursively and output files
        can be written to specified output directory
        """
        _, f_out, status = td.import_dir(self.demo_dir_rec, recursive=True,
                                         out_dir=self.demo_dir)
        self.assertEqual(status, [True for _ in f_out])
        self.assertEqual(len(f_out), 4)
        for file in f_out:
            os.remove(file)


class TestHertz(unittest.TestCase):
    """
    test case methods for tribology functions related to Hertz contact theory
    """

    def test_hertz_mean_pressure_ball(self):
        """
        base test for phertz
        """
        e_eff = th.eeff(210000, 0.3, 210000, 0.3)
        r_eff, r_eff_x, r_eff_y = th.reff(6.35, 6.35, 0, 0)
        p_mean = th.phertz(r_eff, r_eff_x, r_eff_y, e_eff, 10)
        self.assertEqual(round(p_mean), 574)

    def test_hertz_load_carrying(self):
        """
        base test for fhertz
        """
        e_eff = th.eeff(210000, 0.3, 210000, 0.3)
        r_eff, r_eff_x, r_eff_y = th.reff(6.35, 6.35, 0, 0)
        f_crit = th.fhertz(r_eff, r_eff_x, r_eff_y, e_eff, 574)
        self.assertEqual(round(f_crit, 2), 9.99)

    def test_hertz_half_axes(self):
        """
        base test for ahertz
        """
        e_eff = th.eeff(210000, 0.3, 210000, 0.3)
        r_eff, r_eff_x, r_eff_y = th.reff(15, 0, 0, 10)
        ax_a, ax_b, area = th.ahertz(r_eff, r_eff_x, r_eff_y, e_eff,
                                     100)
        rounded = [round(var, 3) for var in (ax_a, ax_b, area)]
        self.assertEqual(rounded, [0.175, 0.227, 0.125])

    def test_approx_hertz_rad(self):
        """
        base test for approx_hertz_rad
        """
        axis = np.asarray([-1, 0, 1])
        prof = np.asarray([0, 1, 0])
        self.assertEqual(round(th.approx_hertz_rad(axis, prof)), 1)
        prof = np.asarray([0, 0, 0])
        self.assertEqual(th.approx_hertz_rad(axis, prof), float('inf'))


class TestP3can(unittest.TestCase):
    """
    integration tests for p3can. load user input templates and check if
    they run without error.
    """
#
#     def test_template01(self):
#         out_dir = p3can('tribology/tests/p3can/' +
#                         'Template01_SingleRowCylindricalRollerBearing.py')
#
#     def test_template03(self):
#         out_dir = p3can('tribology/tests/p3can/' +
#                         'Template03_CylindricalRollerThustBearing.py')
#
#     def test_template05(self):
#         out_dir = p3can('tribology/tests/p3can/' +
#                         'Template05_PinOnDisk.py')
#
#     def test_template06(self):
#         out_dir = p3can('tribology/tests/p3can/' +
#                         'Template06_4Ball.py')
#
    def test_template07(self):
        out_dir = p3can('tribology/tests/p3can/' +
                        'Template07_BallOn3Plates.py')
#
#     def test_template08(self):
#         out_dir = p3can('tribology/tests/p3can/' +
#                         'Template08_RingOnRing.py')


import copy
import os
import re
import time

from Constants import PrintOpts
from check_environment import check_lapack, check_python_environment
from import_user_input import get_user_input
from system_functions import print_it, get_p3can_version, in_d


class Sim:
    # TODO: add class attribute describtion
    """Simulation information"""

    def __init__(self, in_file, out_dir):

        self.ui, self.invalid_input = get_user_input(in_file)

        os.environ["AUTO_PRINT"] = str(self.ui['AUTO_PRINT'])

        print_it("p3can simulation started")
        print_it("this is p3can version {}".format(get_p3can_version()))

        print_it("checking python and fortran libraries")

        check_python_environment()
        check_lapack()

        print_it("setting up simulation")
        print_it("validating user input", PrintOpts.lvl1.value)

        for key, value in self.invalid_input.items():
            print_it(
                "parameter '{} = {}' seems to be invalid. "
                "trying to go on anyway.".format(key, value),
                PrintOpts.lvl2.value)

        self.simulation_type = self.ui['simulation_type']
        self.simulation_name = self.ui['simulation_name']
        self.auto_print = self.ui['auto_print']
        self.auto_plot = self.ui['auto_plot']
        self.auto_report = self.ui['auto_report']
        self.start_time = time.time()

        self.calculation_input = ["simulation_type"]
        self.required_user_input = None
        self.optional_user_input = None
        self.positive_vars = None
        self.res_dir = None
        self.odd_vars = None
        self.alphanumeric_vars = None
        self.parameter_id = None
        self.infl_db_file_hand = None

        self.mk_uniq_res_dir(self.ui['simulation_name'], out_dir)
        print_it("results folder created: {}".format(self.res_dir),
                 PrintOpts.lvl1.value)

        self.input_file_name = self.save_user_input()
        print_it(
            "saved calculation input to file: {}".format(self.input_file_name),
            PrintOpts.lvl1.value)

    def print_time_elapsed(self):
        """Prints time elapsed since start of program"""
        print_it(("time elapsed: " "%d s" % (time.time() - self.start_time)))

    def get_time_elapsed(self):
        """Gets time elapsed since start of program"""
        return time.time() - self.start_time

    def finished(self):
        """Prints total run time at end of program execution"""
        print_it("\np3can finished")
        print_it("you can find all calculation outputs in {}".format(
            self.res_dir.replace('..{}'.format(os.sep), '')))
        self.print_time_elapsed()

    def mk_uniq_res_dir(self, simulation_name, out_dir):
        """Makes unique results folder based on 'simulation_name' and current
        date and time"""
        out_path = os.sep.join([out_dir, 'results/'])
        if os.path.isdir(out_path) is False:
            os.mkdir(out_path)
        valid_file_name = "".join(
            [c for c in simulation_name if re.match(r'\w|-', c)])
        results_folder = out_path + valid_file_name + '--' + time.strftime(
            "%Y-%m-%d--%H-%M-%S")
        unique_results_folder = copy.copy(results_folder)
        counter = 1
        while os.path.isdir('/'.join([os.getcwd(), unique_results_folder])):
            unique_results_folder = results_folder + '-{}'.format(counter)
            counter += 1
        self.res_dir = unique_results_folder
        os.mkdir(self.res_dir)

    def save_user_input(self):
        """Creates text file in results folder that contains the user input
        required for the current simulation. variables that are not required
        for the simulation but were defined by the user are not saved to the
        file"""
        input_file_name = os.sep.join([self.res_dir, "simulation_input.txt"])
        with open(input_file_name, "a") as results_file:
            print_str = "# list of relevant user input.\n" \
                        "# variables that were not used for the calculation " \
                        "are not listed here.\n\n"
            results_file.write(print_str)
            for key, value in self.ui.items():
                if isinstance(value, str):
                    print_str = key + " = '" + value + "'\n"
                else:
                    print_str = key + " = {}\n".format(value)
                results_file.write(print_str)
        return input_file_name

    def mk_uniq_parameter_id(self):
        """Make a unique parameter id (basically one very long string
        containing all inputs) based on the user input. this is used to
        identify unique calculation inputs (if the very long strings are
        identical) and allows to reuse
        influence matrices"""
        self.parameter_id = '-'.join(
            [str(in_d('simulation_type', self.ui, 'x')),
             str(in_d('radial_clearance', self.ui, 'x')),
             str(in_d('radial_clearance', self.ui, 'x')),
             str(in_d('number_balls', self.ui, 'x')),
             str(in_d('number_rollers', self.ui, 'x')),
             str(in_d('number_pins', self.ui, 'x')),
             str(in_d('number_planets', self.ui, 'x')),
             str(in_d('e_cb1', self.ui, 'x')),
             str(in_d('ny_cb1', self.ui, 'x')),
             str(in_d('diameter_cb1', self.ui, 'x')),
             str(in_d('length_cb1', self.ui, 'x')),
             str(in_d('type_profile_cb1', self.ui, 'x')),
             str(in_d('path_profile_cb1', self.ui, 'x')),
             str(in_d('profile_radius_cb1', self.ui, 'x')),
             str(in_d('e_cb2', self.ui, 'x')),
             str(in_d('ny_cb2', self.ui, 'x')),
             str(in_d('diameter_cb2', self.ui, 'x')),
             str(in_d('type_profile_cb2', self.ui, 'x')),
             str(in_d('profile_radius_cb2', self.ui, 'x')),
             str(in_d('length_cb2', self.ui, 'x')),
             str(in_d('path_profile_cb2', self.ui, 'x')),
             str(in_d('e_cb3', self.ui, 'x')),
             str(in_d('ny_cb3', self.ui, 'x')),
             str(in_d('diameter_cb3', self.ui, 'x')),
             str(in_d('type_profile_cb3', self.ui, 'x')),
             str(in_d('length_cb3', self.ui, 'x')),
             str(in_d('profile_radius_cb3', self.ui, 'x')),
             str(in_d('path_profile_cb3', self.ui, 'x')),
             str(in_d('global_force', self.ui, 'x')),
             str(in_d('res_x', self.ui, 'x')),
             str(in_d('res_y', self.ui, 'x')),
             str(in_d('res_pol', self.ui, 'x'))
             ]).replace('/', 'Slash')
        self.ui.update({'parameter_id': self.parameter_id})



import copy
import math
import warnings

import numpy as np

from Constants import Profiles, PrintOpts
from system_functions import exit_program, print_it


def create_2d_profile2(diameter, length, type_profile, x_axis, res_x,
                       path_profile=None, circle_radius=None):
    """creates a 2d profile along x-axis (body length) based on user input"""
    x_profile = np.zeros(res_x)
    file_x_axis = None
    file_x_profile = None
    if type_profile == Profiles.ISO.value:
        # problem is that ISO profile is not defined at the end points of the
        # roller, hence the profile at the end points is calculated based on a
        # slightly shorter x_axis here.
        x_axis_slightly_shorter = copy.deepcopy(x_axis)
        x_axis_slightly_shorter[[0, -1]] = np.multiply([x_axis[0], x_axis[-1]],
                                                       0.99999999)
        x_profile = -0.00035 * diameter * np.log(
            1 / (1 - np.power(((2 * x_axis_slightly_shorter) / length), 2)))
        # x_profile[0] = x_profile[1] * 1.5
        # x_profile[-1] = x_profile[-2] * 1.5

    elif type_profile == Profiles.Circle.value:
        x_profile = np.array(abs(circle_radius) - np.sqrt(
            np.power(circle_radius, 2) - np.power(x_axis, 2)))
        sign_circle_radius = np.sign(circle_radius)
        x_profile = -np.multiply(x_profile, sign_circle_radius)

    elif type_profile == Profiles.File.value:
        content_text_file = []
        try:
            content_text_file = np.loadtxt(path_profile, delimiter="\t")
        except ValueError:
            exit_program(
                "\nexpecting the file '{}' to be a two column tab-delimited "
                ".txt-file"
                "\nwith first column corresponding to length of body and "
                "second column corresponding to "
                "profile data".format(path_profile))

        # get original x-axis and profile data from file and normalize it
        x_axis = np.array(content_text_file[:, 0]) - max(
            np.array(content_text_file[:, 0])) / 2
        file_x_axis = x_axis
        x_profile = np.array(content_text_file[:, 1]) - min(
            content_text_file[:, 1])
        file_x_profile = (x_profile - max(x_profile)) / 1000
        x_profile = x_profile - max(x_profile)
        step = max(1, round(len(content_text_file) / 15000))

        # deactivate filter warnings for polynomial fits
        warnings.simplefilter('ignore', np.RankWarning)

        try:
            print_it("fitting measured profile with 3rd grade polynomial",
                     PrintOpts.lvl1.value)
            start_point_left, start_point_right = fit_profile(x_axis, x_profile,
                                                              step, 3)
            x_axis_new = np.array(
                content_text_file[start_point_left:-start_point_right + 1, 0])
            x_profile_new = np.array(
                x_profile[start_point_left:-start_point_right + 1])
            x_axis_new = x_axis_new - min(x_axis_new)
            x_axis_new -= max(x_axis_new) / 2
            print_it("3rd grade polynomial fit successful",
                     PrintOpts.lvl1.value)
        except ValueError:
            print_it("3rd grade polynomial fit failed", PrintOpts.lvl1.value)
            print_it("fitting measured profile with 2nd grade polynomial",
                     PrintOpts.lvl1.value)
            start_point_left, start_point_right = fit_profile(x_axis, x_profile,
                                                              step, 2)
            x_axis_new = np.array(
                content_text_file[start_point_left:-start_point_right + 1, 0])
            x_profile_new = np.array(
                x_profile[start_point_left:-start_point_right + 1])
            x_axis_new = x_axis_new - min(x_axis_new)
            x_axis_new -= max(x_axis_new) / 2
            print_it("2nd grade polynomial fit successful",
                     PrintOpts.lvl1.value)
        warnings.simplefilter('always', np.RankWarning)

        # generate profile representation and corresponding x-axis based on
        # polynomial fit
        short_x_axis = np.linspace(x_axis_new[0], x_axis_new[-1], res_x)
        z = np.polyfit(x_axis_new, x_profile_new,
                       find_poly(x_axis_new, x_profile_new))
        p = np.poly1d(z)
        interpolated_x_profile = p(short_x_axis)
        x_axis = short_x_axis

        # normalize profile and convert units to millimeter
        x_profile = (interpolated_x_profile - min(interpolated_x_profile))
        x_profile = (x_profile - max(x_profile)) / 1000

    delta_x = abs(x_axis[0] - x_axis[1])
    return x_axis, x_profile, file_x_axis, file_x_profile, delta_x


def revolve_profile2(diameter, x_profile, y_axis, res_x, res_y):
    """creates a 3d profile by revolving self.x_profile around the central
    axis of the body"""
    profile = np.zeros((res_x, res_y))
    x_profile = copy.copy(-x_profile)
    sign_diameter = np.sign(diameter)
    for i in range(res_x):
        for j in range(res_y):
            bracket = math.pow((diameter / 2 - sign_diameter * x_profile[i]),
                               2) - math.pow(y_axis[j], 2)
            if bracket <= 0:
                profile[i, j] = abs(diameter) / 2
            else:
                profile[i, j] = abs(diameter) / 2 - sign_diameter * math.sqrt(
                    bracket)
    if diameter > 0:
        min_body_profile = profile.min()
    else:
        min_body_profile = profile[round(res_x / 2) - 1, round(res_y / 2) - 1]
    profile = -(profile - min_body_profile)
    y_profile = profile[:, math.floor(res_y / 2)]
    return profile, y_profile


def find_poly(x_axis_new, x_profile_new):
    """find the best polynomial fit, either degree 2, 3 or 4"""
    pol_grade = [2, 3, 4]
    z = (np.polyfit(x_axis_new, x_profile_new, item) for item in pol_grade)
    p = list((np.poly1d(item) for item in z))
    profiles = (item(x_axis_new) for item in p)
    diffs = list(sum(abs(item - x_profile_new)) for item in profiles)
    return pol_grade[diffs.index(min(diffs))]


def fit_profile(x_axis, x_profile, step, polyfit_grade):
    """ This function is mainly concerned with correcting the profile to get
    rid of the parts of the profile that do not belong to the raceway, but were
    measured anyway. To see what the function does, remove the comments below;
    you will see a matplotlib plot that is continuously updated while the
    function is running"""
    start_point = -step
    length = len(x_axis)
    half_length = round(length / 2)
    trigger = 0
    diff_triangle = 0
    diff_polynomial = 1
    start_point_left = None
    start_point_right = None
    while diff_polynomial > np.sum(diff_triangle):
        start_point += step
        seventh_length = round((half_length - start_point) / 3.5) + start_point

        # polynomial coefficients for a parabolic fit through the points
        # start_point + seventh_length + half_length
        z_parabolic = np.polyfit(
            x_axis[[start_point, seventh_length - 1, half_length - 1]],
            x_profile[[start_point, seventh_length - 1, half_length - 1]],
            polyfit_grade)
        # polynomial coefficients for a linear fit through the points
        # start_point + seventh_length
        z_triangle1 = np.polyfit(x_axis[[start_point, seventh_length - 1]],
                                 x_profile[[start_point, seventh_length - 1]],
                                 1)
        # polynomial coefficients for a linear fit through the points
        # start_point + half_length
        z_triangle2 = np.polyfit(
            x_axis[[seventh_length, 2 * seventh_length - 1]],
            x_profile[[seventh_length, 2 * seventh_length - 1]], 1)
        z_triangle3 = np.polyfit(x_axis[[2 * seventh_length, half_length - 1]],
                                 x_profile[
                                     [2 * seventh_length, half_length - 1]], 1)

        # evaluate the polynomials
        p_parabolic = np.poly1d(z_parabolic)
        [p_triangle1, p_triangle2, p_triangle3] = (np.poly1d(z) for z in
                                                   [z_triangle1, z_triangle2,
                                                    z_triangle3])

        # create shorter x_axis segments
        short_x_axis = x_axis[start_point:half_length]
        split_short_x_axis_1 = x_axis[start_point:seventh_length - 1]
        split_short_x_axis_2 = x_axis[seventh_length:2 * seventh_length - 1]
        split_short_x_axis_3 = x_axis[2 * seventh_length + 1:half_length - 1]

        # create interpolation data
        fitted_profile = p_parabolic(short_x_axis)
        triangle1 = p_triangle1(split_short_x_axis_1)
        triangle2 = p_triangle2(split_short_x_axis_2)
        triangle3 = p_triangle3(split_short_x_axis_3)

        # calculate difference between original profile and parabolic
        # interpolation
        diff_polynomial = np.sum(
            abs(fitted_profile - x_profile[start_point:half_length]))

        # calculate difference between original profile and triangular
        # interpolation
        combined_triang = np.concatenate((triangle1, triangle2, triangle3),
                                         axis=0)
        if len(combined_triang) > len(
                x_profile[start_point:start_point + len(combined_triang)]):
            diff_triangle = np.absolute(
                combined_triang -
                x_profile[start_point:start_point + len(combined_triang) + 1]
            )
        elif len(combined_triang) < len(
                x_profile[start_point:start_point + len(combined_triang)]
        ):
            diff_triangle = np.absolute(
                combined_triang -
                x_profile[start_point:start_point + len(combined_triang) - 1])
        else:
            diff_triangle = np.absolute(
                combined_triang -
                x_profile[start_point:start_point + len(combined_triang)])

        # uncomment these lines to see what happens during the fitting process:
        # print("diff_poly: " + str(diff_polynomial))
        # print("diff_triangles: " + str(diff_triangle))
        """
        import matplotlib.pyplot as plt
        plt.ion()
        if start_point == 0:
            fig = plt.figure()
            ax = fig.add_subplot(111)
            line1, = ax.plot(x_axis, x_profile, 'k-', 
                             label='original profile data')
            line2, = plt.plot(short_x_axis, fitted_profile, '-.', 
                              label='poly 3rd grade approx.')
            plt.setp(line2, color='r', linewidth=2.0)
            line3, = plt.plot(split_short_x_axis_1, triangle1, '-', 
                              label='linear triangles approx.')
            plt.setp(line3, color='k', linewidth=3.0)
            line4, = plt.plot(split_short_x_axis_2, triangle2, '-')
            plt.setp(line4, color='k', linewidth=3.0)
            line5, = plt.plot(split_short_x_axis_3, triangle3, '-')
            plt.setp(line5, color='k', linewidth=3.0)
            dots, = plt.plot(x_axis[[start_point, seventh_length - 1, 
                                     half_length - 1]], 
                             x_profile[[start_point, seventh_length - 1, 
                                        half_length - 1]],
                             'o', label='interpolation nodes')
            plt.setp(dots, color='k', markersize=10)
            handles, labels = ax.get_legend_handles_labels()
            ax.legend(handles, labels, loc=4)
        else:
            line1.set_xdata(x_axis)
            line1.set_ydata(x_profile)
            line2.set_xdata(short_x_axis)
            line2.set_ydata(fitted_profile)
            line3.set_xdata(split_short_x_axis_1)
            line3.set_ydata(triangle1)
            line4.set_xdata(split_short_x_axis_2)
            line4.set_ydata(triangle2)
            line5.set_xdata(split_short_x_axis_3)
            line5.set_ydata(triangle3)
            dots.set_xdata(x_axis[[start_point, seventh_length - 1, 
                                   half_length - 1]])
            dots.set_ydata(x_profile[[start_point, seventh_length - 1, 
                                      half_length - 1]])
            import time
            time.sleep(0.02)
            fig.canvas.draw()
        """
        if diff_polynomial < np.sum(diff_triangle):
            if trigger == 1:
                start_point_right = start_point
                x_profile = np.flipud(x_profile)
                trigger = 2
            if trigger == 0:
                start_point_left = start_point
                x_profile = np.flipud(x_profile)
                diff_polynomial = 1
                diff_triangle = 0
                trigger = 1
                start_point = -step

    return start_point_left, start_point_right



import copy
import math
import os
import re

import matplotlib.pyplot as plt
import numpy as np
import numpy.polynomial.polynomial as poly
import scipy.io

from BluPrintRollBear import RadialRollBear
from Constants import SubDir, TexTempl, PltOpts, PrintOpts, UnitTex, PreSol
from Constants import Unit
from cartesian_plot_functions import plt_contact, plt_3d, plt_2d_scatt_line, \
    plt_profile_approx, plt_profiles
from generate_latex_output import get_calc_specific_latex_template
from hertz_equations import hertz_displ
from influ_matrix_management import cache_influ_mat, load_influ_mat
from load_distr_cyl_rol_bear import load_distr_cyl_rol_bear
from polar_plot_functions import polplt_scatt_line, polplt_line_line, \
    polplt_line
from solve_half_space import solve_half_space, pre_solve_half_space
from system_functions import make_directory, print_it, print_progress, \
    to_preci, save_to_matlab


class CylRollBear(RadialRollBear):
    """Global bearing data"""

    def __init__(self, number_rollers, roller, ring1, ring2,
                 init_force, radial_clearance=0, res_pol=1,
                 path_roller_slip=None, bearing_name='bearing'):
        super().__init__(bearing_name, number_rollers, roller, ring1, ring2,
                         init_force, radial_clearance, res_pol,
                         path_roller_slip)

        self.ring1.press_zone_len = None
        self.ring2.press_zone_len = None
        self.ring2.eff_footpr_vel = None
        self.ring1.press_pol = None
        self.ring2.press_pol = None
        self.ring2.overroll_t = None
        self.ring2.no_overroll_t = None
        self.ring1.uniq_press = None
        self.ring2.uniq_press = None
        self.slip_data = None

        self.influ_mat_db_1 = None
        self.influ_mat_db_2 = None
        self.roller_norm_forces = None
        self.roller_norm_displ = None
        self.normal_forces = None
        self.norm_displ = None
        self.uniq_norm_displ = None
        self.uniq_norm_forces = None
        self.uniq_force_idx = None

        self.eff_vel_cage = None
        self.pol_coords_slip = None

        self.calc_polar_coordinates()

    def calc_polar_coordinates(self):
        """Set-up a polar coordinate system for the inner/outer ring"""
        if (self.res_pol % self.num_rollers) is not 0:
            self.res_pol = int(
                math.ceil(self.res_pol / self.num_rollers) * self.num_rollers)
            print_it("polar resolution ('res_pol') changed to {}".format(
                self.res_pol), PrintOpts.lvl1.value)

        self.pol_ax = np.linspace(0, 2 * math.pi - 2 * math.pi / self.res_pol,
                                  self.res_pol)
        self.num_roller_pos = int(self.res_pol / self.num_rollers)
        self.d_phi = 2 * math.pi / self.num_rollers
        self.dd_phi = 2 * math.pi / self.res_pol

        self.phi_mat = np.zeros((self.num_roller_pos, self.num_rollers))
        for i in range(self.num_roller_pos):
            self.phi_mat[i, :] = np.linspace(i * self.dd_phi,
                                             2 * math.pi - self.d_phi + i * self.dd_phi,
                                             self.num_rollers)

    def calc_load_distribution(self, ui=None, res_dir=None):
        """Calculate the load distribution for all roller positions by calling
        the function 'load_distr_cyl_rol_bear' while changing the polar
        position of the rollers with respect to the inner/outer ring"""
        print_it(
            'calculating {} load distributions'.format(self.num_roller_pos))

        # initialise variables
        count = 0
        x_profile = abs(self.roller.x_profile + self.ring1.x_profile)
        self.roller_norm_forces = np.zeros(
            (self.num_roller_pos, self.num_rollers))
        self.roller_norm_displ = np.zeros(
            (self.num_roller_pos, self.num_rollers))

        # get all load distributions for all roller positions
        for roller_pos in range(self.num_roller_pos):
            self.roller_norm_forces[roller_pos, :], self.roller_norm_displ[
                                                    roller_pos, :] = \
                load_distr_cyl_rol_bear(self.phi_mat[roller_pos, :],
                                        self.num_rollers, self.roller.length,
                                        self.roller.res_x, x_profile,
                                        self.roller.x_axis, self.rad_clear,
                                        self.init_force)
            count = print_progress(count, self.num_roller_pos)
        self.normal_forces = np.transpose(self.roller_norm_forces).reshape(
            (self.roller_norm_forces.size, 1))
        self.norm_displ = np.transpose(self.roller_norm_displ).reshape(
            (self.roller_norm_displ.size, 1))

        # get unique normal forces and unique normal displacements
        self.uniq_norm_forces, self.uniq_force_idx = np.unique(
            np.round(self.roller_norm_forces), return_inverse=True)
        uniq_norm_displ = np.zeros(len(self.uniq_norm_forces))
        for i in range(len(self.uniq_norm_forces)):
            uniq_norm_displ[i] = np.mean(
                self.norm_displ[self.uniq_force_idx == i])
        self.uniq_norm_displ = sorted(np.absolute(uniq_norm_displ) / 10)

        # save load distribution data to matlab database
        dat_dict = {'roller_positions': self.num_roller_pos,
                    'rollers': self.num_rollers,
                    'roller_normal_forces': self.roller_norm_forces,
                    'uniq_norm_forces': self.uniq_norm_forces,
                    'normal_forces': self.normal_forces,
                    'polar_coordinates': self.phi_mat}
        save_to_matlab(dat_dict, res_dir, 'load-distribution')

    def calc_contact_pressure(self, ui=None, res_dir=None):
        """For each unique normal roller force, calculate the contact pressure
        between inner ring and roller and outer ring and roller"""

        # initialise variables
        print_it('calculating pressure distributions')
        count = 0
        num_uniq_forces = self.uniq_norm_forces.shape[0]
        self.ring1.uniq_press = np.zeros(
            (self.roller.res_x, self.roller.res_y, num_uniq_forces))
        self.ring2.uniq_press = np.zeros(
            (self.roller.res_x, self.roller.res_y, num_uniq_forces))

        # iterate over unique normal forces, starting with highest normal force
        # at end of array
        for unique_norm_force_idx in range(num_uniq_forces - 1, -1, -1):
            if self.uniq_norm_forces[unique_norm_force_idx] > 0:
                norm_force = self.uniq_norm_forces[unique_norm_force_idx]
                norm_displ = self.uniq_norm_displ[unique_norm_force_idx]
                # start with pressure distribution for highest normal force
                # since this determines grid size and
                # influence matrix for all other calculations
                if unique_norm_force_idx == num_uniq_forces - 1:
                    self.solve_highest_pressure(ui, res_dir, norm_force)
                    print_it('solving for {} pressure distributions'.format(
                        2 * num_uniq_forces), PrintOpts.lvl1.value)
                # solve for pressure distributions for other normal forces,
                # reuse influence matrix
                else:
                    self.ring1.loc_press, self.influ_mat_db_1 = \
                        solve_half_space(self.roller.profile,
                                         self.ring1.profile, self.roller.x_axis,
                                         self.roller.y_axis, self.roller.res_x,
                                         self.roller.res_y, self.roller.delta_x,
                                         self.roller.delta_y, self.roller.e,
                                         self.ring1.e, self.roller.ny,
                                         self.ring1.ny, norm_force, res_dir,
                                         influ_mat_db=self.influ_mat_db_1,
                                         print_prog=False,
                                         init_displ=norm_displ)

                    self.ring2.loc_press, self.influ_mat_db_2 = \
                        solve_half_space(self.roller.profile,
                                         self.ring2.profile, self.roller.x_axis,
                                         self.roller.y_axis, self.roller.res_x,
                                         self.roller.res_y, self.roller.delta_x,
                                         self.roller.delta_y, self.roller.e,
                                         self.ring2.e, self.roller.ny,
                                         self.ring2.ny, norm_force, res_dir,
                                         influ_mat_db=self.influ_mat_db_2,
                                         print_prog=False,
                                         init_displ=norm_displ)
                # store pressure data
                self.ring1.uniq_press[:, :,
                unique_norm_force_idx] = self.ring1.loc_press
                self.ring2.uniq_press[:, :,
                unique_norm_force_idx] = self.ring2.loc_press
            count = print_progress(print_progress(count, 2 * num_uniq_forces),
                                   2 * num_uniq_forces)
        self.reassign_pressure_results()
        self.save_pressure_to_matlab(res_dir)
        cache_influ_mat(ui, [self.influ_mat_db_1, self.influ_mat_db_2], res_dir)

    def get_grid_size(self, ui, res_dir, uniq_norm_force):
        """Determine grid size by running (quick) simulation with simplified
        contact bodies. Calculate pressure only for contact between outer ring
        and roller since the contact area will be larger than for the case of
        inner ring and roller."""
        print_it('determining grid size', PrintOpts.lvl1.value)
        self.roller.simple_clone()
        self.roller.clone.make_profile(PreSol.res_x.value, PreSol.res_y.value,
                                       self.init_force)
        self.ring2.simple_clone()
        self.ring2.clone.make_slave_to(self.roller.clone)

        init_displ = hertz_displ(self.roller.clone.e, self.ring2.clone.e,
                                 self.roller.clone.ny, self.ring2.clone.ny,
                                 self.roller.clone.r_hertz_x,
                                 self.roller.clone.r_hertz_y,
                                 self.ring2.clone.r_hertz_x,
                                 self.ring2.clone.r_hertz_y, uniq_norm_force)
        too_many_elements_in_contact = 1
        contact_width = 0.05
        while too_many_elements_in_contact != 0:
            self.roller.clone.make_profile(self.roller.clone.res_x,
                                           self.roller.clone.res_y,
                                           self.init_force,
                                           contact_width=contact_width)
            self.ring2.clone.make_slave_to(self.roller.clone)

            pressure, init_displ = \
                pre_solve_half_space(self.roller.clone.profile,
                                     self.ring2.clone.profile,
                                     self.roller.clone.x_axis,
                                     self.roller.clone.y_axis,
                                     self.roller.clone.res_x,
                                     self.roller.clone.res_y,
                                     self.roller.clone.delta_x,
                                     self.roller.clone.delta_y,
                                     self.roller.clone.e,
                                     self.ring2.clone.e, self.roller.clone.ny,
                                     self.ring2.clone.ny, uniq_norm_force,
                                     init_displ=init_displ, print_prog=False)

            pressure_elements = sum(
                pressure[math.floor(self.roller.clone.res_y / 2), :] > 0)
            too_many_elements_in_contact = self.roller.clone.res_y - \
                                           pressure_elements - 2
            contact_width += -np.sign(
                too_many_elements_in_contact) * contact_width / 25

        self.roller.make_profile(self.roller.res_x, self.roller.res_y,
                                 self.init_force, contact_width=contact_width)
        self.ring1.make_slave_to(self.roller)
        self.ring2.make_slave_to(self.roller)
        return init_displ

    def solve_highest_pressure(self, ui, res_dir, uniq_norm_force):
        """Solve for pressure distribution for highest normal force"""
        init_displ = self.get_grid_size(ui, res_dir, uniq_norm_force)
        [self.influ_mat_db_1, self.influ_mat_db_2] = load_influ_mat(ui, res_dir,
                                                                    2)

        self.ring1.loc_press, self.influ_mat_db_1 = \
            solve_half_space(self.roller.profile, self.ring1.profile,
                             self.roller.x_axis, self.roller.y_axis,
                             self.roller.res_x, self.roller.res_y,
                             self.roller.delta_x, self.roller.delta_y,
                             self.roller.e, self.ring1.e, self.roller.ny,
                             self.ring1.ny, uniq_norm_force, res_dir,
                             print_prog=False, influ_mat_db=self.influ_mat_db_1,
                             init_displ=init_displ)

        self.ring2.loc_press, self.influ_mat_db_2 = \
            solve_half_space(self.roller.profile, self.ring2.profile,
                             self.roller.x_axis, self.roller.y_axis,
                             self.roller.res_x, self.roller.res_y,
                             self.roller.delta_x, self.roller.delta_y,
                             self.roller.e, self.ring2.e, self.roller.ny,
                             self.ring2.ny, uniq_norm_force, res_dir,
                             print_prog=False, influ_mat_db=self.influ_mat_db_2,
                             init_displ=init_displ)

    def reassign_pressure_results(self):
        """Extract and reassign pressure data to allow for easier storage and
        access"""
        idx_mat = np.reshape(self.uniq_force_idx,
                             (self.num_roller_pos, self.num_rollers))
        self.ring1.press_pol = np.zeros(
            (self.roller.res_x, self.roller.res_y, self.res_pol))
        self.ring2.press_pol = copy.copy(self.ring1.press_pol)
        self.ring1.press = np.zeros((self.roller.res_x, self.roller.res_y,
                                     self.num_roller_pos, self.num_rollers))
        self.ring2.press = np.zeros((self.roller.res_x, self.roller.res_y,
                                     self.num_roller_pos, self.num_rollers))
        self.ring1.max_press = np.zeros((self.res_pol, self.roller.res_x))
        self.ring2.max_press = np.zeros((self.res_pol, self.roller.res_x))
        loop_count = 0
        for j in range(self.num_rollers):
            for i in range(self.num_roller_pos):
                self.ring1.press[:, :, i, j] = self.ring1.uniq_press[:, :,
                                               idx_mat[i, j]]
                self.ring2.press[:, :, i, j] = self.ring2.uniq_press[:, :,
                                               idx_mat[i, j]]
                self.ring1.max_press[loop_count, :] = np.amax(
                    self.ring1.press[:, :, i, j], axis=1)
                self.ring2.max_press[loop_count, :] = np.amax(
                    self.ring2.press[:, :, i, j], axis=1)
                self.ring1.press_pol[:, :, loop_count] = self.ring1.uniq_press[
                                                         :, :, idx_mat[i, j]]
                self.ring2.press_pol[:, :, loop_count] = self.ring2.uniq_press[
                                                         :, :, idx_mat[i, j]]
                loop_count += 1

    def calc_kinematics(self, rot_vel1, rot_vel2, ui=None, res_dir=None):
        """Calculate bearing kinematics based on rotational velocities of
        contact bodies"""
        print_it("calculating kinematics")
        self.rot_vel1 = rot_vel1
        self.rot_vel2 = rot_vel2
        self.ring1.omega = rot_vel1 / 60
        self.ring2.omega = rot_vel2 / 60
        self.eff_rot_vel = self.rot_vel1 - self.rot_vel2
        self.eff_omega = self.eff_rot_vel / 60
        self.eff_omega_cage = self.eff_omega / 2

        self.ring1.eff_raceway_vel = self.ring1.diameter * math.pi * \
                                     self.eff_omega
        self.ring2.eff_raceway_vel = 0
        self.eff_vel_cage = (
                            self.ring1.eff_raceway_vel +
                            self.ring2.eff_raceway_vel) / 2
        self.roller.eff_raceway_vel = \
            math.pi * self.roller.diameter * (
            self.mean_diameter / self.roller.diameter) * self.eff_omega / 2
        self.roller.actual_raceway_vel = np.zeros(
            (self.res_pol, self.roller.res_x))
        self.roller.rel_vel = np.zeros((self.res_pol, self.roller.res_x))

        if self.path_roller_slip is None:
            self.roller.actual_raceway_vel[:, :] = self.roller.eff_raceway_vel
        else:
            self.calc_slip_data()

        self.calc_overroll_times()

    def calc_slip_data(self):
        """Load, fit and interpolate splip data from user input file"""
        self.slip_data = []
        with open(self.path_roller_slip, 'r') as f:
            for line in f:
                self.slip_data.append(
                    [float(data_point) for data_point in line.split()])
        self.pol_coords_slip = np.linspace(0, 2 * math.pi, len(self.slip_data))
        slip_fit = poly.polyfit(self.pol_coords_slip, self.slip_data, 17)
        self.roller.slip = poly.polyval(self.pol_ax, slip_fit)[0, :]
        self.roller.slip[np.where(self.roller.slip > 1)[0]] = 1

        for i in range(len(self.pol_ax)):
            self.roller.actual_raceway_vel[i, :] = \
                self.roller.eff_raceway_vel * self.roller.slip[i]
            self.roller.rel_vel[i, :] = \
                self.roller.eff_raceway_vel -\
                self.roller.actual_raceway_vel[i, :]

    def calc_overroll_times(self):
        """Calculate the duration of each overrolling cycle for roller,
        inner ring and outer ring"""
        polar_points_under_load = -1 * np.ones(self.res_pol)
        for i in range(self.res_pol):
            if sum(sum(self.ring2.press_pol[:, :, i])) != 0:
                polar_points_under_load[i] = 1
        circ_ratio_loaded = sum(polar_points_under_load == 1) / self.res_pol
        circ_ratio_unloaded = sum(polar_points_under_load == -1) / self.res_pol

        self.ring1.eff_footpr_vel = 2 * math.pi * (
            self.ring1.diameter / 2) * self.eff_omega_cage
        self.ring2.eff_footpr_vel = 2 * math.pi * (
            self.ring2.diameter / 2) * self.eff_omega_cage
        self.ring1.press_zone_len = (self.ring1.press_pol > 0).sum(
            1) * self.ring1.delta_y
        self.ring2.press_zone_len = (self.ring2.press_pol > 0).sum(
            1) * self.ring2.delta_y

        # case inner ring: if inner ring is standing still
        if self.rot_vel1 == 0:
            self.ring1.overroll_t_incr = \
                self.roller.delta_y / self.ring1.eff_footpr_vel
            self.ring1.no_overroll_t = np.divide(
                ((2 * math.pi * (self.ring1.diameter / 2)) / self.num_rollers -
                 self.ring1.press_zone_len), self.ring1.eff_footpr_vel)
        # case inner ring: if inner ring is standing still
        else:
            t_ratio_loaded_ring1 = circ_ratio_loaded
            t_ratio_unloaded_ring1 = circ_ratio_unloaded
            self.ring1.overroll_t_incr = \
                self.roller.delta_y / self.ring1.eff_footpr_vel
            self.ring1.no_overroll_t = np.divide(
                ((2 * math.pi * (
                self.ring1.diameter / 2)) / self.num_rollers -
                self.ring1.press_zone_len),
                self.ring1.eff_footpr_vel) * t_ratio_loaded_ring1 + 2 * \
                math.pi * (self.ring1.diameter / 2) * t_ratio_unloaded_ring1 / \
                self.ring1.eff_footpr_vel

        # case outer ring: if outer ring is standing still
        if self.rot_vel2 == 0:
            self.ring2.overroll_t_incr = \
                self.roller.delta_y / self.ring2.eff_footpr_vel
            self.ring2.no_overroll_t = np.divide(
                ((2 * math.pi * (self.ring2.diameter / 2)) / self.num_rollers -
                 self.ring2.press_zone_len), self.ring2.eff_footpr_vel)
        # case outer ring: if outer ring is rotating
        else:
            t_ratio_loaded_ring2 = circ_ratio_loaded
            t_ratio_unloaded_ring2 = circ_ratio_unloaded
            self.ring2.overroll_t_incr = \
                self.roller.delta_y / self.ring2.eff_footpr_vel
            self.ring2.no_overroll_t = np.divide(
                ((2 * math.pi * (self.ring2.diameter / 2)) / self.num_rollers -
                self.ring2.press_zone_len),
                self.ring2.eff_footpr_vel) * t_ratio_loaded_ring2 + \
                2 * math.pi * (
                self.ring2.diameter / 2) * t_ratio_unloaded_ring2 / \
                self.ring2.eff_footpr_vel

    def calc_pv(self, ui=None, res_dir=None):
        """Calculate product of local maximum pressure and local maximum
        relative velocity"""
        print_it("calculating pv_rel")
        self.ring1.pv = np.absolute(
            np.multiply(self.ring1.max_press, self.roller.rel_vel) / 1000)
        self.ring2.pv = np.absolute(
            np.multiply(self.ring2.max_press, self.roller.rel_vel) / 1000)
        dat_dict = dict(x_axis=self.roller.x_axis,
                        pv_rel_ir_3d=self.ring1.pv,
                        pv_rel_ir_2d=np.amax(self.ring1.pv, axis=1),
                        pv_rel_or_3d=self.ring2.pv,
                        pv_rel_or_2d=np.amax(self.ring2.pv, axis=1),
                        axis_pol=self.pol_ax)
        save_to_matlab(dat_dict, res_dir, 'pv_rel_ring2')

    def calc_e_akin(self, ui=None, res_dir=None):
        """"Calculate the kinetic friction energy accumulation in W per m^2"""
        print_it("calculating e_a,kin")

        pv_loc_ring1 = np.multiply(self.ring1.press_pol.sum(1),
                                   self.roller.rel_vel.transpose())
        pv_loc_ring2 = np.multiply(self.ring2.press_pol.sum(1),
                                   self.roller.rel_vel.transpose())

        if self.rot_vel2 == 0:
            self.ring2.e_akin = np.absolute(
                np.divide(np.multiply(pv_loc_ring2, self.ring2.overroll_t_incr),
                          self.ring2.no_overroll_t)) / 1000
        else:
            self.ring2.e_akin = np.absolute(np.divide(
                np.multiply(pv_loc_ring2.sum(1), self.ring2.overroll_t_incr),
                self.ring2.no_overroll_t.sum(1))) / 1000
        if self.rot_vel1 == 0:
            self.ring1.e_akin = np.absolute(
                np.divide(np.multiply(pv_loc_ring1, self.ring2.overroll_t_incr),
                          self.ring1.no_overroll_t)) / 1000
        else:
            self.ring1.e_akin = np.absolute(np.divide(
                np.multiply(pv_loc_ring1.sum(1), self.ring1.overroll_t_incr),
                self.ring1.no_overroll_t.sum(1))) / 1000

    def plot_it(self, ui=None, res_dir=None):
        """Orchestrate output plot generation"""
        print_it("plotting results and finishing up")
        plt_profiles(res_dir)
        self.plt_profiles_contacts(res_dir)
        self.plt_pol_slip(res_dir)
        self.plt_load_distr(res_dir)
        self.plt_pres(res_dir)
        self.plt_pv(res_dir)
        self.plt_eakin(res_dir)
        self.generate_summary_figure(res_dir)

    def save_pressure_to_matlab(self, res_dir):
        dat_dict = dict(contact_pressures_IR=self.ring1.press,
                        contact_pressures_OR=self.ring2.press,
                        x_axis=self.roller.x_axis,
                        y_axis=self.roller.y_axis,
                        roller_number=np.linspace(1, self.num_rollers,
                                                  self.num_rollers),
                        roller_position=np.linspace(1, self.num_roller_pos,
                                                    self.num_roller_pos))
        save_to_matlab(dat_dict, res_dir, 'pressure-field')

    def generate_latex_output(self, calc_spec_tex_file_handle, sim, ui=None,
                              res_dir=None):
        """Generate the calculation-specific part of the latex output file"""
        av_press_ring1 = np.mean(
            self.ring1.press[:, :, 0, 0][self.ring1.press[:, :, 0, 0] > 0])
        av_press_ring2 = np.mean(
            self.ring2.press[:, :, 0, 0][self.ring2.press[:, :, 0, 0] > 0])
        out_data = [
            ('pv_rel inner ring, max.', to_preci(np.amax(self.ring1.pv), 4),
             UnitTex.pvrel.value, 'unverified'),
            ('pv_rel outer ring, max.', to_preci(np.amax(self.ring2.pv), 4),
             UnitTex.pvrel.value, 'unverified'),
            ('e_akin inner ring, max.',
             to_preci(np.amax([self.ring1.e_akin]), 4),
             UnitTex.eakin.value, 'unverified'),
            ('e_akin outer ring, max.',
             to_preci(np.amax([self.ring2.e_akin]), 4),
             UnitTex.eakin.value, 'unverified'),
            ('pressure inner ring, max. av.', to_preci(av_press_ring1, 4),
             UnitTex.pressure.value, 'unverified'),
            ('pressure outer ring, max. av.', to_preci(av_press_ring2, 4),
             UnitTex.pressure.value, 'unverified'),
            (
            'normal force, max.', to_preci(np.amax(self.roller_norm_forces), 4),
            UnitTex.force.value, 'unverified'),
            ('pressure inner ring, max. max.',
             to_preci(np.amax(self.ring1.press[:, :, 0, 0]), 4),
             UnitTex.pressure.value, 'unverified'),
            ('pressure outer ring, max. max.',
             to_preci(np.amax(self.ring2.press[:, :, 0, 0]), 4),
             UnitTex.pressure.value, 'unverified'),
            ('contact area inner ring, max.',
             to_preci(self.roller.get_area(self.ring1.press[:, :, 0, 0]), 4),
             UnitTex.area.value, 'unverified'),
            ('contact area outer ring, max.',
             to_preci(self.roller.get_area(self.ring2.press[:, :, 0, 0]), 4),
             UnitTex.area.value, 'unverified')
            ]

        scale_factor_ir = 0.5 + 0.1 * (self.rot_vel1 != 0)
        scale_factor_or = 0.5 + 0.1 * (self.rot_vel2 != 0)

        table_calc_summary = []
        for key, value, unit, status in sorted(out_data):
            table_calc_summary.append(
                (re.sub('_', '\_', key), value, unit, status))

        latex_variables = {'contact_plot1': '{}{}contact1.png'.format(
            SubDir.tex_figs_rel_to_tex_file.value, '/'),
                           'contact_plot2': '{}{}contact2.png'.format(
                               SubDir.tex_figs_rel_to_tex_file.value, '/'),
                           'scale_factor_ir': scale_factor_ir,
                           'scale_factor_or': scale_factor_or,
                           'table_calc_summary': table_calc_summary,
                           'load_plot1': '{}{}load1.png'.format(
                               SubDir.tex_figs_rel_to_tex_file.value, '/'),
                           'pressure_plot1': '{}{}pressure1.png'.format(
                               SubDir.tex_figs_rel_to_tex_file.value, '/'),
                           'pressure_plot2': '{}{}pressure2.png'.format(
                               SubDir.tex_figs_rel_to_tex_file.value, '/'),
                           'slip_plot1': '{}{}slip1.png'.format(
                               SubDir.tex_figs_rel_to_tex_file.value, '/'),
                           'energy_plot1': '{}{}energy1.png'.format(
                               SubDir.tex_figs_rel_to_tex_file.value, '/'),
                           'energy_plot2': '{}{}energy2.png'.format(
                               SubDir.tex_figs_rel_to_tex_file.value, '/'),
                           'energy_plot3': '{}{}energy3.png'.format(
                               SubDir.tex_figs_rel_to_tex_file.value, '/')
                           }

        template_calc_specific = get_calc_specific_latex_template(
            TexTempl.CylindricalRollerBearing.value, sim)
        with open(calc_spec_tex_file_handle, 'w') as f:
            f.write(template_calc_specific.render(latex_variables))

    def generate_latex_figures(self, ui=None, res_dir=None):
        """Append system-specific output data to latex file"""
        max_press_field_ring1 = self.ring1.press[:, :, 0, 0]
        max_press_field_ring2 = self.ring2.press[:, :, 0, 0]

        plt_profile_approx(res_dir, SubDir.tex_figs.value)
        plt_contact(self.roller, self.ring1, PltOpts.DDD.value, res_dir,
                    SubDir.tex_figs.value, 'contact1')
        plt_contact(self.roller, self.ring2, PltOpts.DDD.value, res_dir,
                    SubDir.tex_figs.value, 'contact2')
        polplt_scatt_line(self.pol_coords_slip, self.slip_data, self.pol_ax,
                          self.roller.slip, 'roller slip',
                          'roller slip', res_dir, SubDir.tex_figs.value,
                          'slip1')

        plt_3d(self.roller.x_axis, self.roller.y_axis, max_press_field_ring1,
               self.roller.x_label, self.roller.y_label,
               'Pressure in MPa', 'pressure at highest normal force', res_dir,
               SubDir.tex_figs.value,
               'pressure1')

        plt_3d(self.roller.x_axis, self.roller.y_axis, max_press_field_ring2,
               self.roller.x_label, self.roller.y_label,
               'Pressure in MPa', 'pressure at highest normal force', res_dir,
               SubDir.tex_figs.value,
               'pressure2')

        polplt_scatt_line(np.append(self.phi_mat[0, :], self.phi_mat[0, 0]),
                          np.append(self.roller_norm_forces[0, :],
                                    self.roller_norm_forces[0, 0]),
                          np.append(self.phi_mat[0, :], self.phi_mat[0, 0]),
                          np.append(self.roller_norm_forces[0, :],
                                    self.roller_norm_forces[0, 0]),
                          'normal force in N', 'load distribution', res_dir,
                          SubDir.tex_figs.value,
                          'load1')

        plot_axis_pol = np.linspace(-math.pi,
                                    math.pi - 2 * math.pi / self.res_pol,
                                    int(self.res_pol))
        plot_data_ring1 = np.concatenate(
            (self.ring1.pv[round(self.res_pol / 2) - 1:-1, :],
             self.ring2.pv[0:round(self.res_pol / 2), :]), axis=0)
        plot_data_ring2 = np.concatenate(
            (self.ring2.pv[round(self.res_pol / 2) - 1:-1, :],
             self.ring2.pv[0:round(self.res_pol / 2), :]), axis=0)

        polplt_line_line(plot_axis_pol,
                         plot_data_ring1[:, round(self.roller.res_x / 2) - 1],
                         plot_axis_pol,
                         plot_data_ring2[:, round(self.roller.res_x / 2) - 1],
                         'pv in {}'.format(Unit.pvrel.value),
                         'pv_rel inner (max) and outer ring',
                         'inner ring (max)', 'outer ring',
                         res_dir, SubDir.tex_figs.value, 'energy1')

        if self.rot_vel1 == 0:
            plot_data = np.concatenate((self.ring1.e_akin.transpose()[
                                        round(self.res_pol / 2) - 1:-1, :],
                                        self.ring1.e_akin.transpose()[
                                        0:round(self.res_pol / 2), :]), axis=0)
            polplt_line(plot_axis_pol,
                        plot_data[:, round(self.roller.res_x / 2) - 1],
                        'e_a,kin_rel max in {}'.format(Unit.eakin.value),
                        'e_a,kin max inner ring',
                        res_dir, SubDir.tex_figs.value, 'energy2')
        else:
            plt_2d_scatt_line(self.roller.x_axis, self.ring1.e_akin,
                              self.roller.x_axis, self.ring1.e_akin,
                              self.roller.x_label,
                              'average e_a,kin in {}'.format(Unit.eakin.value),
                              'average e_a,kin inner ring', res_dir,
                              SubDir.tex_figs.value, 'energy2')
        if self.rot_vel2 == 0:
            plot_data = np.concatenate(
                (self.ring2.e_akin.transpose()[round(self.res_pol / 2) - 1:-1,
                 :],
                 self.ring2.e_akin.transpose()[0:round(self.res_pol / 2), :]),
                axis=0)
            polplt_line(plot_axis_pol,
                        plot_data[:, round(self.roller.res_x / 2) - 1],
                        'e_a,kin_rel max in {}'.format(Unit.eakin.value),
                        'e_a,kin max outer ring',
                        res_dir, SubDir.tex_figs.value, 'energy3')
        else:
            plt_2d_scatt_line(self.roller.x_axis, self.ring2.e_akin,
                              self.roller.x_axis, self.ring2.e_akin,
                              self.roller.x_label,
                              'average e_a,kin in {}'.format(Unit.eakin.value),
                              'average e_a,kin outer ring', res_dir,
                              SubDir.tex_figs.value, 'energy3')

    def plt_profiles_contacts(self, res_dir):
        """Generate plots of profiles and contacts"""
        print_it("plotting contacts", PrintOpts.lvl1.value)
        plt_contact(self.roller, self.ring1, PltOpts.DD.value, res_dir,
                    SubDir.contacts.value)
        plt_contact(self.roller, self.ring1, PltOpts.DDD.value, res_dir,
                    SubDir.contacts.value)
        plt_contact(self.roller, self.ring2, PltOpts.DD.value, res_dir,
                    SubDir.contacts.value)
        plt_contact(self.roller, self.ring2, PltOpts.DDD.value, res_dir,
                    SubDir.contacts.value)

    def plt_load_distr(self, res_dir):
        """Generate plots of load distribution"""
        print_it("plotting load distributions", PrintOpts.lvl1.value)
        for roller_position in range(self.num_roller_pos):
            roller_forces = self.roller_norm_forces[roller_position, :]
            roller_forces_for_plot = np.append(roller_forces, roller_forces[0])
            phi_plot = np.append(self.phi_mat[roller_position, :],
                                 self.phi_mat[roller_position, 0])
            polplt_scatt_line(phi_plot, roller_forces_for_plot, phi_plot,
                              roller_forces_for_plot, 'normal force in N',
                              ('load distribution position ' + str(
                                  roller_position)), res_dir,
                              SubDir.load_distr.value, (
                              'load_distribution_position_' + str(
                                  roller_position)),
                              self.roller_norm_forces[0, 0] * 1.1)
            print_progress(roller_position, self.num_roller_pos)

        polplt_scatt_line(self.pol_ax, self.normal_forces, self.pol_ax,
                          self.normal_forces, 'normal force in N',
                          'roller normal force', res_dir,
                          SubDir.load_distr.value, 'normal-force-over-position')

    def plt_pol_slip(self, res_dir):
        """Generate plot of polar roller slip"""
        polplt_scatt_line(self.pol_coords_slip, self.slip_data, self.pol_ax,
                          self.roller.slip, 'roller slip',
                          'roller slip', res_dir, SubDir.energy.value,
                          'roller.slip')

    def plt_pres(self, res_dir):
        """Generate contact pressure plots"""
        print_it("plotting pressure distributions", PrintOpts.lvl1.value)
        press_count = 0
        z_limit = np.amax(
            self.ring1.uniq_press[:, :, self.uniq_norm_forces.shape[0] - 1])
        for uniq_norm_force in range(self.uniq_norm_forces.shape[0] - 1, -1,
                                     -1):
            plt_3d(self.roller.x_axis, self.roller.y_axis,
                   self.ring1.uniq_press[:, :, uniq_norm_force],
                   self.roller.x_label, self.roller.y_label, 'pressure in MPa',
                   ('contact_pressure_ir_' + str(int(press_count))), res_dir,
                   SubDir.pressures.value,
                   ('contact_pressure_ir_' + str(int(press_count))),
                   [0, z_limit])
            plt_3d(self.roller.x_axis, self.roller.y_axis,
                   self.ring2.uniq_press[:, :, uniq_norm_force],
                   self.roller.x_label, self.roller.y_label, 'pressure in MPa',
                   ('contact_pressure_or_' + str(int(press_count))), res_dir,
                   SubDir.pressures.value,
                   ('contact_pressure_or_' + str(int(press_count))),
                   [0, z_limit])
            press_count = print_progress(press_count,
                                         self.uniq_norm_forces.shape[0])

    def plt_pv(self, res_dir):
        """Plot pv"""
        plt_ax_pol = np.linspace(-math.pi, math.pi - 2 * math.pi / self.res_pol,
                                 int(self.res_pol))
        plot_data_ring1 = np.concatenate(
            (self.ring1.pv[round(self.res_pol / 2) - 1:-1, :],
             self.ring2.pv[0:round(self.res_pol / 2), :]), axis=0)
        plot_data_ring2 = np.concatenate(
            (self.ring2.pv[round(self.res_pol / 2) - 1:-1, :],
             self.ring2.pv[0:round(self.res_pol / 2), :]), axis=0)
        plt_3d(plt_ax_pol, self.roller.x_axis, plot_data_ring1,
               'polar ring coordinate', 'roller length in mm',
               'pv in {}'.format(Unit.pvrel.value), 'pv inner ring', res_dir,
               SubDir.energy.value, 'max_pv_inner_ring',
               None, PltOpts.azim.value)
        plt_3d(plt_ax_pol, self.roller.x_axis, plot_data_ring2,
               'polar ring coordinate', 'roller length in mm',
               'pv in {}'.format(Unit.pvrel.value), 'pv outer ring', res_dir,
               SubDir.energy.value, 'pv_outer_ring',
               None, PltOpts.azim.value)

        polplt_line_line(plt_ax_pol,
                         plot_data_ring1[:, round(self.roller.res_x / 2) - 1],
                         plt_ax_pol,
                         plot_data_ring2[:, round(self.roller.res_x / 2) - 1],
                         'pv in {}'.format(Unit.pvrel.value),
                         'pv_rel inner (max) and outer ring',
                         'inner ring (max)', 'outer ring', res_dir,
                         SubDir.energy.value, 'pv-rel-max')
        scipy.io.savemat(res_dir +
                         '{}{}{}pv_rel_ring2.mat'.format(
                             os.sep,
                             SubDir.matlab.value,
                             os.sep
                         ),
                         dict(plot_axis_pol=plt_ax_pol,
                              plot_axis_x=self.roller.x_axis,
                              plot_data_pv_rel_ir_3d=plot_data_ring1,
                              plot_data_pv_rel_ir_2d=np.amax(plot_data_ring1,
                                                             axis=1),
                              plot_data_pv_rel_or_3d=plot_data_ring2,
                              plot_data_pv_rel_or_2d=np.amax(plot_data_ring2,
                                                             axis=1),
                              axis_pol=self.pol_ax,
                              axis_x=self.roller.x_axis,
                              data_pv_rel_or=self.ring2.pv))

    def plt_eakin(self, res_dir):
        """Plot kinetic friction energy accumulation"""
        print_it("plotting energy figures", PrintOpts.lvl1.value)
        # e_a,kin
        plt_ax_pol = np.linspace(-math.pi, math.pi - 2 * math.pi / self.res_pol,
                                 int(self.res_pol))
        if self.rot_vel1 == 0:
            plot_data = np.concatenate((self.ring1.e_akin.transpose()[
                                        round(self.res_pol / 2) - 1:-1, :],
                                        self.ring1.e_akin.transpose()[
                                        0:round(self.res_pol / 2), :]), axis=0)
            plt_3d(plt_ax_pol, self.roller.x_axis, plot_data,
                   'polar ring coordinate', 'roller length in mm',
                   'e_a,kin in {}'.format(Unit.eakin.value),
                   'e_a,kin inner ring', res_dir, SubDir.energy.value,
                   'e-a-kin-inner-ring', None, PltOpts.azim.value)

            polplt_line(plt_ax_pol,
                        plot_data[:, round(self.roller.res_x / 2) - 1],
                        'e_a,kin_rel max in {}'.format(Unit.eakin.value),
                        'e_a,kin max inner ring', res_dir,
                        SubDir.energy.value, 'max-e-a-kin-inner-ring')
        else:
            plt_2d_scatt_line(self.roller.x_axis, self.ring1.e_akin,
                              self.roller.x_axis, self.ring1.e_akin,
                              self.roller.x_label,
                              'average e_a,kin in {}'.format(Unit.eakin.value),
                              'average e_a,kin inner ring', res_dir,
                              SubDir.energy.value, 'average-e-a-kin-inner-ring')

        if self.rot_vel2 == 0:
            plot_data = np.concatenate((self.ring2.e_akin.transpose()[
                                        round(self.res_pol / 2) - 1:-1, :],
                                        self.ring2.e_akin.transpose()[
                                        0:round(self.res_pol / 2), :]), axis=0)
            plt_3d(plt_ax_pol, self.roller.x_axis, plot_data,
                   'polar ring coordinate', 'roller length in mm',
                   'e_a,kin in {}'.format(Unit.eakin.value),
                   'e_a,kin outer ring', res_dir, SubDir.energy.value,
                   'e-a-kin-outer-ring', None, PltOpts.azim.value)

            polplt_line(plt_ax_pol,
                        plot_data[:, round(self.roller.res_x / 2) - 1],
                        'e_a,kin_rel max in {}'.format(Unit.eakin.value),
                        'e_a,kin max outer ring', res_dir,
                        SubDir.energy.value, 'max-e-a-kin-outer-ring')
        else:
            plt_2d_scatt_line(self.roller.x_axis, self.ring2.e_akin,
                              self.roller.x_axis, self.ring2.e_akin,
                              self.roller.x_label,
                              'average e_a,kin in {}'.format(Unit.eakin.value),
                              'average e_a,kin outer ring', res_dir,
                              SubDir.energy.value, 'average-e-a-kin-outer-ring')

    def generate_summary_figure(self, res_dir):
        """Generate figure summarising the simulation results in a polar plot"""
        plot_axis_pol = np.linspace(0, 2 * math.pi - 2 * math.pi / self.res_pol,
                                    int(self.res_pol))
        pv_max_ring1 = self.ring1.pv
        eakin_dat = self.ring1.e_akin.transpose()
        press_dat = self.ring1.max_press
        force_dat = self.normal_forces
        slip_dat = self.roller.slip

        plt.rc('grid', linewidth=0.5, linestyle=':')
        plt.figure(facecolor='w')
        ax = plt.subplot(111, polar=True)

        ax.grid(True)

        # plot pv
        r = np.amax(pv_max_ring1, axis=1) / np.amax(pv_max_ring1)
        theta = plot_axis_pol
        leg = 'pv'
        ax.plot(theta, r, 'k-', label=leg)
        ind = round(len(theta) * 0.17)
        thisr, thistheta = r[ind], theta[ind]
        ax.annotate(leg,
                    xy=(thistheta, thisr),  # theta, radius
                    xytext=(thistheta, thisr),  # fraction, fraction
                    textcoords='data',
                    arrowprops=dict(facecolor='black', shrink=0.05),
                    horizontalalignment='left',
                    verticalalignment='bottom',
                    )

        # plot e_akin
        r = np.amax(eakin_dat, axis=1) / np.amax(eakin_dat)
        leg = '$\mathregular{e_{a,kin}}$'
        ax.plot(theta, r, 'k--', label=leg)
        ind = round(len(theta) * 0.03)
        thisr, thistheta = r[ind], theta[ind]
        ax.annotate(leg,
                    xy=(thistheta, thisr),  # theta, radius
                    xytext=(thistheta, thisr),  # fraction, fraction
                    textcoords='data',
                    arrowprops=dict(facecolor='black', shrink=0.05),
                    horizontalalignment='right',
                    verticalalignment='top',
                    )

        # plot pressure
        r = np.amax(press_dat, axis=1) / np.amax(press_dat)
        leg = '$\mathregular{p_{max}}$'
        ax.plot(theta, r, 'k-.', label=leg)
        ind = round(len(theta) / 1.1)
        thisr, thistheta = r[ind], theta[ind]
        ax.annotate(leg,
                    xy=(thistheta, thisr),  # theta, radius
                    xytext=(thistheta, thisr),  # fraction, fraction
                    textcoords='data',
                    arrowprops=dict(facecolor='black', shrink=0.05),
                    horizontalalignment='right',
                    verticalalignment='bottom',
                    )

        # plot force
        r = np.amax(force_dat, axis=1) / np.amax(force_dat)
        leg = '$\mathregular{F_{N}}$'
        ax.plot(theta, r, 'k--', label=leg)
        ind = round(len(theta) / 1.05)
        thisr, thistheta = r[ind], theta[ind]
        ax.annotate(leg,
                    xy=(thistheta, thisr),  # theta, radius
                    xytext=(thistheta, thisr),  # fraction, fraction
                    textcoords='data',
                    arrowprops=dict(facecolor='black', shrink=0.05),
                    horizontalalignment='left',
                    verticalalignment='bottom',
                    )

        # plot slip
        r = slip_dat / np.amax(slip_dat)
        leg = 'roller slip'
        ax.plot(theta, r, 'k--', label=leg)
        ind = round(len(theta) * 0.79)
        thisr, thistheta = r[ind], theta[ind]
        ax.annotate(leg,
                    xy=(thistheta, thisr),  # theta, radius
                    xytext=(thistheta, thisr),  # fraction, fraction
                    textcoords='data',
                    arrowprops=dict(facecolor='black', shrink=0.05),
                    horizontalalignment='left',
                    verticalalignment='bottom',
                    )

        # plt.legend(loc='upper center')
        ax.set_theta_zero_location("S")
        ax.set_rlabel_position(112.5)
        label_position = ax.get_rlabel_position()
        r_max = 1.1
        r_label = 'normalised value'
        ax.text(np.radians(label_position + 15), r_max / 2, r_label,
                rotation=22.5, ha='center', va='center')
        ax.set_rmax(r_max)
        make_directory(res_dir, SubDir.pub_figs.value)
        file_handle = res_dir + os.sep + SubDir.pub_figs.value + os.sep + \
                      'normalised-inner-ring' + '.png'
        plt.savefig(file_handle, bbox_inches='tight', dpi=300)
        plt.close()
